@@ -49,7 +49,7 @@ QString QgsSplitVectorLayerAlgorithm::groupId() const
 
 QString QgsSplitVectorLayerAlgorithm::shortHelpString() const
 {
-  return QObject::tr( "Splits input vector layer into multiple layers by specified unique ID field." )
+  return QObject::tr( "This algorithm splits input vector layer into multiple layers by specified unique ID field." )
          + QStringLiteral( "\n\n" )
          + QObject::tr( "Each of the layers created in the output folder contains all features from "
                         "the input layer with the same value for the specified attribute. The number "
@@ -57,6 +57,12 @@ QString QgsSplitVectorLayerAlgorithm::shortHelpString() const
                         "specified attribute." );
 }
 
+QString QgsSplitVectorLayerAlgorithm::shortDescription() const
+{
+  return QObject::tr( "Splits a vector layer into multiple layers, each containing "
+                      "all the features with the same value for a specified attribute." );
+}
+
 QgsSplitVectorLayerAlgorithm *QgsSplitVectorLayerAlgorithm::createInstance() const
 {
   return new QgsSplitVectorLayerAlgorithm();
@@ -64,15 +70,13 @@ QgsSplitVectorLayerAlgorithm *QgsSplitVectorLayerAlgorithm::createInstance() con
 
 void QgsSplitVectorLayerAlgorithm::initAlgorithm( const QVariantMap & )
 {
-  addParameter( new QgsProcessingParameterFeatureSource( QStringLiteral( "INPUT" ), QObject::tr( "Input layer" ), QList<int>() << static_cast< int >( Qgis::ProcessingSourceType::Vector ) ) );
-  addParameter( new QgsProcessingParameterField( QStringLiteral( "FIELD" ), QObject::tr( "Unique ID field" ),
-                QVariant(), QStringLiteral( "INPUT" ) ) );
-  std::unique_ptr< QgsProcessingParameterBoolean > prefixFieldParam = std::make_unique< QgsProcessingParameterBoolean >( QStringLiteral( "PREFIX_FIELD" ),
-      QObject::tr( "Add field prefix to file names" ), true );
+  addParameter( new QgsProcessingParameterFeatureSource( QStringLiteral( "INPUT" ), QObject::tr( "Input layer" ), QList<int>() << static_cast<int>( Qgis::ProcessingSourceType::Vector ) ) );
+  addParameter( new QgsProcessingParameterField( QStringLiteral( "FIELD" ), QObject::tr( "Unique ID field" ), QVariant(), QStringLiteral( "INPUT" ) ) );
+  auto prefixFieldParam = std::make_unique<QgsProcessingParameterBoolean>( QStringLiteral( "PREFIX_FIELD" ), QObject::tr( "Add field prefix to file names" ), true );
   addParameter( prefixFieldParam.release() );
 
   const QStringList options = QgsVectorFileWriter::supportedFormatExtensions();
-  auto fileTypeParam = std::make_unique < QgsProcessingParameterEnum >( QStringLiteral( "FILE_TYPE" ), QObject::tr( "Output file type" ), options, false, 0, true );
+  auto fileTypeParam = std::make_unique<QgsProcessingParameterEnum>( QStringLiteral( "FILE_TYPE" ), QObject::tr( "Output file type" ), options, false, 0, true );
   fileTypeParam->setFlags( fileTypeParam->flags() | Qgis::ProcessingParameterFlag::Advanced );
   addParameter( fileTypeParam.release() );
 
@@ -82,7 +86,7 @@ void QgsSplitVectorLayerAlgorithm::initAlgorithm( const QVariantMap & )
 
 QVariantMap QgsSplitVectorLayerAlgorithm::processAlgorithm( const QVariantMap &parameters, QgsProcessingContext &context, QgsProcessingFeedback *feedback )
 {
-  std::unique_ptr< QgsProcessingFeatureSource > source( parameterAsSource( parameters, QStringLiteral( "INPUT" ), context ) );
+  std::unique_ptr<QgsProcessingFeatureSource> source( parameterAsSource( parameters, QStringLiteral( "INPUT" ), context ) );
   if ( !source )
     throw QgsProcessingException( invalidSourceError( parameters, QStringLiteral( "INPUT" ) ) );
 
@@ -108,7 +112,7 @@ QVariantMap QgsSplitVectorLayerAlgorithm::processAlgorithm( const QVariantMap &p
   const QgsCoordinateReferenceSystem crs = source->sourceCrs();
   const Qgis::WkbType geometryType = source->wkbType();
   const int fieldIndex = fields.lookupField( fieldName );
-  const QSet< QVariant > uniqueValues = source->uniqueValues( fieldIndex );
+  const QSet<QVariant> uniqueValues = source->uniqueValues( fieldIndex );
   QString baseName = outputDir + QDir::separator();
 
   if ( parameterAsBool( parameters, QStringLiteral( "PREFIX_FIELD" ), context ) )
@@ -122,7 +126,7 @@ QVariantMap QgsSplitVectorLayerAlgorithm::processAlgorithm( const QVariantMap &p
   int count = 0;
   QgsFeature feat;
   QStringList outputLayers;
-  std::unique_ptr< QgsFeatureSink > sink;
+  std::unique_ptr<QgsFeatureSink> sink;
 
   for ( auto it = uniqueValues.constBegin(); it != uniqueValues.constEnd(); ++it )
   {
@@ -157,6 +161,7 @@ QVariantMap QgsSplitVectorLayerAlgorithm::processAlgorithm( const QVariantMap &p
         throw QgsProcessingException( writeFeatureError( sink.get(), parameters, QStringLiteral( "OUTPUT" ) ) );
       count += 1;
     }
+    sink->finalize();
 
     feedback->pushInfo( QObject::tr( "Added %n feature(s) to layer", nullptr, count ) );
     outputLayers << fileName;
