@@ -26,6 +26,7 @@
 #include <QTextCodec>
 
 #include "qgsclipboard.h"
+#include "moc_qgsclipboard.cpp"
 #include "qgsfeature.h"
 #include "qgsfeaturestore.h"
 #include "qgsfields.h"
@@ -42,6 +43,8 @@
 #include "qgsapplication.h"
 #include "qgsvectortilelayer.h"
 #include "qgsvectorlayerutils.h"
+#include "qgsmemoryproviderutils.h"
+#include "qgsmessagebar.h"
 
 #include <nlohmann/json.hpp>
 
@@ -75,7 +78,7 @@ void QgsClipboard::replaceWithCopyOf( QgsVectorTileLayer *src )
   // things are a bit tricky for vector tile features, as each will have different fields
   // so we build a "super set" of fields first, and then make sure each feature has that superset present
 
-  const QList< QgsFeature > selectedFeatures = src->selectedFeatures();
+  const QList<QgsFeature> selectedFeatures = src->selectedFeatures();
   QgsFields supersetFields;
   for ( const QgsFeature &feature : selectedFeatures )
   {
@@ -122,7 +125,7 @@ void QgsClipboard::replaceWithCopyOf( QgsFeatureStore &featureStore, QgsVectorLa
 
 void QgsClipboard::generateClipboardText( QString &textContent, QString &htmlContent ) const
 {
-  CopyFormat format = QgsSettings().enumValue( QStringLiteral( "qgis/copyFeatureFormat" ),  AttributesWithWKT );
+  CopyFormat format = QgsSettings().enumValue( QStringLiteral( "qgis/copyFeatureFormat" ), AttributesWithWKT );
 
   textContent.clear();
   htmlContent.clear();
@@ -147,7 +150,7 @@ void QgsClipboard::generateClipboardText( QString &textContent, QString &htmlCon
       // first do the field names
       if ( ( format == AttributesWithWKB ) || ( format == AttributesWithWKT ) )
       {
-        const QLatin1String geometryHeading = format == AttributesWithWKB  ? QLatin1String( "wkb_geom" ) : QLatin1String( "wkt_geom" );
+        const QLatin1String geometryHeading = format == AttributesWithWKB ? QLatin1String( "wkb_geom" ) : QLatin1String( "wkt_geom" );
         // only include the "wkx_geom" field IF we have other fields -- otherwise it's redundant and we should just set the clipboard to WKT/WKB text directly
         if ( !mFeatureFields.isEmpty() )
         {
@@ -306,20 +309,18 @@ QgsFeatureList QgsClipboard::stringToFeatureList( const QString &string, const Q
     return features;
 
   // Poor man's csv parser
-  bool isInsideQuotes {false};
+  bool isInsideQuotes { false };
   QgsAttributes attrs;
   QgsGeometry geom;
   QString attrVal;
-  bool isFirstLine {string.startsWith( QLatin1String( "wkt_geom" ) )};
+  bool isFirstLine { string.startsWith( QLatin1String( "wkt_geom" ) ) };
   // it seems there is no other way to check for header
-  const bool hasHeader{string.startsWith( QLatin1String( "wkt_geom" ) )};
+  const bool hasHeader { string.startsWith( QLatin1String( "wkt_geom" ) ) };
   QgsGeometry geometry;
-  bool setFields {fields.isEmpty()};
+  bool setFields { fields.isEmpty() };
   QgsFields fieldsFromClipboard;
 
-  auto parseFunc = [ & ]( const QChar & c )
-  {
-
+  auto parseFunc = [&]( const QChar &c ) {
     // parse geom only if it wasn't successfully set before
     if ( geometry.isNull() )
     {
@@ -330,7 +331,7 @@ QgsFeatureList QgsClipboard::stringToFeatureList( const QString &string, const Q
     {
       if ( attrVal != QLatin1String( "wkt_geom" ) ) // ignore this one
       {
-        fieldsFromClipboard.append( QgsField{attrVal, QMetaType::Type::QString } );
+        fieldsFromClipboard.append( QgsField { attrVal, QMetaType::Type::QString } );
       }
     }
     else // ... or value
@@ -347,7 +348,7 @@ QgsFeatureList QgsClipboard::stringToFeatureList( const QString &string, const Q
       }
       else
       {
-        QgsFeature feature{setFields ? fieldsFromClipboard : fields};
+        QgsFeature feature { setFields ? fieldsFromClipboard : fields };
         feature.setGeometry( geometry );
         if ( hasHeader || !geometry.isNull() )
         {
@@ -542,12 +543,16 @@ void QgsClipboard::setText( const QString &text )
 
 bool QgsClipboard::hasFormat( const QString &mimeType ) const
 {
-  return QApplication::clipboard()->mimeData()->hasFormat( mimeType );
+  if ( const QMimeData *data = QApplication::clipboard()->mimeData() )
+    return data->hasFormat( mimeType );
+  return false;
 }
 
 QByteArray QgsClipboard::data( const QString &mimeType ) const
 {
-  return QApplication::clipboard()->mimeData()->data( mimeType );
+  if ( const QMimeData *data = QApplication::clipboard()->mimeData() )
+    return data->data( mimeType );
+  return QByteArray();
 }
 
 QgsFields QgsClipboard::fields() const
@@ -566,6 +571,117 @@ QgsMapLayer *QgsClipboard::layer() const
     return nullptr;
 }
 
+std::unique_ptr<QgsVectorLayer> QgsClipboard::pasteToNewMemoryVector( QgsMessageBar *messageBar )
+{
+  const QgsFields fields = QgsClipboard::fields();
+
+  // Decide geometry type from features, switch to multi type if at least one multi is found
+  QMap<Qgis::WkbType, int> typeCounts;
+  const QgsFeatureList features = copyOf( fields );
+  for ( const QgsFeature &feature : features )
+  {
+    if ( !feature.hasGeometry() )
+      continue;
+
+    const Qgis::WkbType type = feature.geometry().wkbType();
+
+    if ( type == Qgis::WkbType::Unknown || type == Qgis::WkbType::NoGeometry )
+      continue;
+
+    if ( QgsWkbTypes::isSingleType( type ) )
+    {
+      if ( typeCounts.contains( QgsWkbTypes::multiType( type ) ) )
+      {
+        typeCounts[QgsWkbTypes::multiType( type )] = typeCounts[QgsWkbTypes::multiType( type )] + 1;
+      }
+      else
+      {
+        typeCounts[type] = typeCounts[type] + 1;
+      }
+    }
+    else if ( QgsWkbTypes::isMultiType( type ) )
+    {
+      if ( typeCounts.contains( QgsWkbTypes::singleType( type ) ) )
+      {
+        // switch to multi type
+        typeCounts[type] = typeCounts[QgsWkbTypes::singleType( type )];
+        typeCounts.remove( QgsWkbTypes::singleType( type ) );
+      }
+      typeCounts[type] = typeCounts[type] + 1;
+    }
+  }
+
+  const Qgis::WkbType wkbType = !typeCounts.isEmpty() ? typeCounts.constBegin().key() : Qgis::WkbType::NoGeometry;
+
+  if ( features.isEmpty() )
+  {
+    // should not happen
+    if ( messageBar )
+      messageBar->pushMessage( tr( "Paste features" ), tr( "No features in clipboard." ), Qgis::MessageLevel::Info );
+    return nullptr;
+  }
+  else if ( typeCounts.size() > 1 )
+  {
+    QString typeName = wkbType != Qgis::WkbType::NoGeometry ? QgsWkbTypes::displayString( wkbType ) : QStringLiteral( "none" );
+    if ( messageBar )
+      messageBar->pushMessage( tr( "Paste features" ), tr( "Multiple geometry types found, features with geometry different from %1 will be created without geometry." ).arg( typeName ), Qgis::MessageLevel::Info );
+  }
+
+  std::unique_ptr<QgsVectorLayer> layer( QgsMemoryProviderUtils::createMemoryLayer( QStringLiteral( "pasted_features" ), QgsFields(), wkbType, crs() ) );
+
+  if ( !layer->isValid() || !layer->dataProvider() )
+  {
+    if ( messageBar )
+      messageBar->pushMessage( tr( "Paste features" ), tr( "Cannot create new layer." ), Qgis::MessageLevel::Warning );
+    return nullptr;
+  }
+
+  layer->startEditing();
+  for ( const QgsField &f : QgsClipboard::fields() )
+  {
+    QgsDebugMsgLevel( QStringLiteral( "field %1 (%2)" ).arg( f.name(), QVariant::typeToName( f.type() ) ), 2 );
+    if ( !layer->addAttribute( f ) )
+    {
+      if ( messageBar )
+        messageBar->pushMessage( tr( "Paste features" ), tr( "Cannot create field %1 (%2,%3), falling back to string type" ).arg( f.name(), f.typeName(), QVariant::typeToName( f.type() ) ), Qgis::MessageLevel::Warning );
+
+      // Fallback to string
+      QgsField strField { f };
+      strField.setType( QMetaType::Type::QString );
+      if ( !layer->addAttribute( strField ) )
+      {
+        if ( messageBar )
+          messageBar->pushMessage( tr( "Paste features" ), tr( "Cannot create field %1 (%2,%3)" ).arg( strField.name(), strField.typeName(), QVariant::typeToName( strField.type() ) ), Qgis::MessageLevel::Critical );
+        return nullptr;
+      }
+    }
+  }
+
+  QgsFeatureList convertedFeatures { QgsVectorLayerUtils::makeFeaturesCompatible( features, layer.get() ) };
+
+  // Convert attributes
+  for ( auto it = convertedFeatures.begin(); it != convertedFeatures.end(); ++it )
+  {
+    for ( int idx = 0; idx < layer->fields().count() && idx < it->attributeCount(); ++idx )
+    {
+      QVariant attr = it->attribute( idx );
+      //if convertCompatible fails, it will replace attr with an appropriate null QVariant
+      //so we're calling setAttribute regardless, covering cases like 'Autogenerate' or 'nextVal()' on int fields.
+      layer->fields().at( idx ).convertCompatible( attr );
+      it->setAttribute( idx, attr );
+    }
+  }
+
+  if ( !layer->addFeatures( convertedFeatures ) || !layer->commitChanges() )
+  {
+    QgsDebugError( QStringLiteral( "Cannot add features or commit changes" ) );
+    return nullptr;
+  }
+
+  QgsDebugMsgLevel( QStringLiteral( "%1 features pasted to temporary scratch layer" ).arg( layer->featureCount() ), 2 );
+  return layer;
+}
+
 void QgsClipboard::systemClipboardChanged()
 {
   if ( mIgnoreNextSystemClipboardChange )
