@@ -167,13 +167,6 @@ QgsRectangle QgsMeshLayer::extent() const
   }
 }
 
-QString QgsMeshLayer::providerType() const
-{
-  QGIS_PROTECT_QOBJECT_THREAD_ACCESS
-
-  return mProviderKey;
-}
-
 bool QgsMeshLayer::supportsEditing() const
 {
   QGIS_PROTECT_QOBJECT_THREAD_ACCESS
@@ -253,6 +246,30 @@ QString QgsMeshLayer::loadDefaultStyle( bool &resultFlag )
   return QgsMapLayer::loadDefaultStyle( resultFlag );
 }
 
+bool QgsMeshLayer::removeDatasets( const QString &name )
+{
+  const int index = mDatasetGroupStore->indexFromGroupName( name );
+
+  if ( index == -1 )
+  {
+    return false;
+  }
+
+  const QgsMeshDatasetGroupMetadata groupMetadata =  datasetGroupMetadata( index );
+
+  mDatasetGroupStore->removeDatasetGroup( index );
+
+  if ( mExtraDatasetUri.contains( groupMetadata.uri() ) )
+  {
+    mExtraDatasetUri.removeOne( groupMetadata.uri() );
+  }
+
+  resetDatasetGroupTreeItem();
+
+  emit dataSourceChanged();
+  return true;
+}
+
 bool QgsMeshLayer::addDatasets( const QString &path, const QDateTime &defaultReferenceTime )
 {
   QGIS_PROTECT_QOBJECT_THREAD_ACCESS
@@ -394,7 +411,7 @@ QgsMeshRendererSettings QgsMeshLayer::rendererSettings() const
   return mRendererSettings;
 }
 
-void QgsMeshLayer::setRendererSettings( const QgsMeshRendererSettings &settings )
+void QgsMeshLayer::setRendererSettings( const QgsMeshRendererSettings &settings, const bool repaint )
 {
   QGIS_PROTECT_QOBJECT_THREAD_ACCESS
 
@@ -409,7 +426,11 @@ void QgsMeshLayer::setRendererSettings( const QgsMeshRendererSettings &settings
     emit activeVectorDatasetGroupChanged( mRendererSettings.activeVectorDatasetGroup() );
 
   emit rendererChanged();
-  triggerRepaint();
+
+  if ( repaint )
+  {
+    triggerRepaint();
+  }
 }
 
 QgsMeshTimeSettings QgsMeshLayer::timeSettings() const
@@ -872,38 +893,6 @@ void QgsMeshLayer::setDatasetGroupTreeRootItem( QgsMeshDatasetGroupTreeItem *roo
   updateActiveDatasetGroups();
 }
 
-int QgsMeshLayer::closestEdge( const QgsPointXY &point, double searchRadius, QgsPointXY &projectedPoint ) const
-{
-  QGIS_PROTECT_QOBJECT_THREAD_ACCESS
-
-  const QgsRectangle searchRectangle( point.x() - searchRadius, point.y() - searchRadius, point.x() + searchRadius, point.y() + searchRadius );
-  const QgsTriangularMesh *mesh = triangularMesh();
-  // search for the closest edge in search area from point
-  const QList<int> edgeIndexes = mesh->edgeIndexesForRectangle( searchRectangle );
-  int selectedIndex = -1;
-  if ( mesh->contains( QgsMesh::Edge ) &&
-       mDataProvider->isValid() )
-  {
-    double sqrMaxDistFromPoint = pow( searchRadius, 2 );
-    for ( const int edgeIndex : edgeIndexes )
-    {
-      const QgsMeshEdge &edge = mesh->edges().at( edgeIndex );
-      const QgsMeshVertex &vertex1 = mesh->vertices()[edge.first];
-      const QgsMeshVertex &vertex2 = mesh->vertices()[edge.second];
-      QgsPointXY projPoint;
-      const double sqrDist = point.sqrDistToSegment( vertex1.x(), vertex1.y(), vertex2.x(), vertex2.y(), projPoint, 0 );
-      if ( sqrDist < sqrMaxDistFromPoint )
-      {
-        selectedIndex = edgeIndex;
-        projectedPoint = projPoint;
-        sqrMaxDistFromPoint = sqrDist;
-      }
-    }
-  }
-
-  return selectedIndex;
-}
-
 QgsMeshDatasetIndex QgsMeshLayer::staticVectorDatasetIndex( int group ) const
 {
   QGIS_PROTECT_QOBJECT_THREAD_ACCESS
@@ -928,14 +917,49 @@ void QgsMeshLayer::setTemporalMatchingMethod( const QgsMeshDataProviderTemporalC
   mTemporalProperties->setMatchingMethod( matchingMethod );
 }
 
-QgsPointXY QgsMeshLayer::snapOnVertex( const QgsPointXY &point, double searchRadius )
+int QgsMeshLayer::closestEdge( const QgsPointXY &point, double searchRadius, QgsPointXY &projectedPoint ) const
+{
+  QGIS_PROTECT_QOBJECT_THREAD_ACCESS
+
+  const QgsRectangle searchRectangle( point.x() - searchRadius, point.y() - searchRadius, point.x() + searchRadius, point.y() + searchRadius );
+  const QgsTriangularMesh *mesh = triangularMesh();
+  // search for the closest edge in search area from point
+  const QList<int> edgeIndexes = mesh->edgeIndexesForRectangle( searchRectangle );
+  int selectedIndex = -1;
+  projectedPoint = QgsPointXY();
+  if ( mesh->contains( QgsMesh::Edge ) &&
+       mDataProvider->isValid() )
+  {
+    double sqrMaxDistFromPoint = pow( searchRadius, 2 );
+    for ( const int edgeIndex : edgeIndexes )
+    {
+      const QgsMeshEdge &edge = mesh->edges().at( edgeIndex );
+      const QgsMeshVertex &vertex1 = mesh->vertices()[edge.first];
+      const QgsMeshVertex &vertex2 = mesh->vertices()[edge.second];
+      QgsPointXY projPoint;
+      const double sqrDist = point.sqrDistToSegment( vertex1.x(), vertex1.y(), vertex2.x(), vertex2.y(), projPoint, 0 );
+      if ( sqrDist < sqrMaxDistFromPoint )
+      {
+        selectedIndex = edgeIndex;
+        projectedPoint = projPoint;
+        sqrMaxDistFromPoint = sqrDist;
+      }
+    }
+  }
+
+  return selectedIndex;
+}
+
+int QgsMeshLayer::closestVertex( const QgsPointXY &point, double searchRadius, QgsPointXY &projectedPoint ) const
 {
   QGIS_PROTECT_QOBJECT_THREAD_ACCESS
 
   const QgsTriangularMesh *mesh = triangularMesh();
-  QgsPointXY exactPosition;
+  int selectedIndex = -1;
+  projectedPoint = QgsPointXY();
   if ( !mesh )
-    return exactPosition;
+    return selectedIndex;
+
   const QgsRectangle rectangle( point.x() - searchRadius, point.y() - searchRadius, point.x() + searchRadius, point.y() + searchRadius );
   double maxDistance = searchRadius;
   //attempt to snap on edges's vertices
@@ -950,12 +974,14 @@ QgsPointXY QgsMeshLayer::snapOnVertex( const QgsPointXY &point, double searchRad
     if ( dist1 < maxDistance )
     {
       maxDistance = dist1;
-      exactPosition = vertex1;
+      projectedPoint = vertex1;
+      selectedIndex = edge.first;
     }
     if ( dist2 < maxDistance )
     {
       maxDistance = dist2;
-      exactPosition = vertex2;
+      projectedPoint = vertex2;
+      selectedIndex = edge.second;
     }
   }
 
@@ -971,32 +997,25 @@ QgsPointXY QgsMeshLayer::snapOnVertex( const QgsPointXY &point, double searchRad
       if ( dist < maxDistance )
       {
         maxDistance = dist;
-        exactPosition = vertex;
+        projectedPoint = vertex;
+        selectedIndex = face.at( i );
       }
     }
   }
 
-  return exactPosition;
+  return selectedIndex;
 }
 
-QgsPointXY QgsMeshLayer::snapOnEdge( const QgsPointXY &point, double searchRadius )
-{
-  QGIS_PROTECT_QOBJECT_THREAD_ACCESS
-
-  QgsPointXY projectedPoint;
-  closestEdge( point, searchRadius, projectedPoint );
-
-  return projectedPoint;
-}
-
-QgsPointXY QgsMeshLayer::snapOnFace( const QgsPointXY &point, double searchRadius )
+int QgsMeshLayer::closestFace( const QgsPointXY &point, double searchRadius, QgsPointXY &projectedPoint ) const
 {
   QGIS_PROTECT_QOBJECT_THREAD_ACCESS
 
   const QgsTriangularMesh *mesh = triangularMesh();
-  QgsPointXY centroidPosition;
+  int selectedIndex = -1;
+  projectedPoint = QgsPointXY();
   if ( !mesh )
-    return centroidPosition;
+    return selectedIndex;
+
   const QgsRectangle rectangle( point.x() - searchRadius, point.y() - searchRadius, point.x() + searchRadius, point.y() + searchRadius );
   double maxDistance = std::numeric_limits<double>::max();
 
@@ -1011,11 +1030,12 @@ QgsPointXY QgsMeshLayer::snapOnFace( const QgsPointXY &point, double searchRadiu
     if ( dist < maxDistance )
     {
       maxDistance = dist;
-      centroidPosition = centroid;
+      projectedPoint = centroid;
+      selectedIndex = nativefaceIndex;
     }
   }
 
-  return centroidPosition;
+  return selectedIndex;
 }
 
 void QgsMeshLayer::resetDatasetGroupTreeItem()
@@ -1468,16 +1488,25 @@ QgsPointXY QgsMeshLayer::snapOnElement( QgsMesh::ElementType elementType, const
 {
   QGIS_PROTECT_QOBJECT_THREAD_ACCESS
 
+  QgsPointXY projectedPoint;
+  closestElement( elementType, point, searchRadius, projectedPoint );
+  return projectedPoint;
+}
+
+int QgsMeshLayer::closestElement( QgsMesh::ElementType elementType, const QgsPointXY &point, double searchRadius, QgsPointXY &projectedPoint ) const
+{
+  QGIS_PROTECT_QOBJECT_THREAD_ACCESS
+
   switch ( elementType )
   {
     case QgsMesh::Vertex:
-      return snapOnVertex( point, searchRadius );
+      return closestVertex( point, searchRadius, projectedPoint );
     case QgsMesh::Edge:
-      return snapOnEdge( point, searchRadius );
+      return closestEdge( point, searchRadius, projectedPoint );
     case QgsMesh::Face:
-      return snapOnFace( point, searchRadius );
+      return closestFace( point, searchRadius, projectedPoint );
   }
-  return QgsPointXY(); // avoid warnings
+  return -1;
 }
 
 QList<int> QgsMeshLayer::selectVerticesByExpression( QgsExpression expression )
@@ -1664,6 +1693,98 @@ QgsMapLayerRenderer *QgsMeshLayer::createMapRenderer( QgsRenderContext &renderer
   return new QgsMeshLayerRenderer( this, rendererContext );
 }
 
+QgsMeshDatasetIndex QgsMeshLayer::activeScalarDatasetIndex( QgsRenderContext &rendererContext )
+{
+  if ( rendererContext.isTemporal() )
+    return activeScalarDatasetAtTime( rendererContext.temporalRange(), mRendererSettings.activeScalarDatasetGroup() );
+  else
+    return staticScalarDatasetIndex( mRendererSettings.activeScalarDatasetGroup() );
+}
+
+bool QgsMeshLayer::minimumMaximumActiveScalarDataset( const QgsRectangle &extent, const QgsMeshDatasetIndex &datasetIndex, double &min, double &max )
+{
+
+  if ( extent.isNull() || !this->extent().intersects( extent ) )
+    return false;
+
+  QgsTriangularMesh *tMesh = triangularMesh();
+
+  if ( ! tMesh )
+  {
+    return false;
+  }
+
+  QVector<double> scalarDatasetValues;
+  const QgsMeshDatasetGroupMetadata metadata = datasetGroupMetadata( datasetIndex.group() );
+
+  if ( !metadata.isScalar() )
+  {
+    return false;
+  }
+
+  QgsMeshDatasetGroupMetadata::DataType scalarDataType = QgsMeshLayerUtils::datasetValuesType( metadata.dataType() );
+
+  if ( !datasetIndex.isValid() )
+  {
+    return false;
+  }
+
+  // populate scalar values
+  const int count = QgsMeshLayerUtils::datasetValuesCount( mNativeMesh.get(), scalarDataType );
+  const QgsMeshDataBlock vals = QgsMeshLayerUtils::datasetValues(
+                                  this,
+                                  datasetIndex,
+                                  0,
+                                  count );
+
+  if ( vals.isValid() )
+  {
+    // vals could be scalar or vectors, for contour rendering we want always magnitude
+    scalarDatasetValues = QgsMeshLayerUtils::calculateMagnitudes( vals );
+  }
+  else
+  {
+    scalarDatasetValues = QVector<double>( count, std::numeric_limits<double>::quiet_NaN() );
+  }
+
+  QList<int> intersectedFacesIndices = tMesh->faceIndexesForRectangle( extent );
+
+  if ( intersectedFacesIndices.isEmpty() )
+  {
+    return false;
+  }
+
+  min = std::numeric_limits<double>::max();
+  max = -std::numeric_limits<double>::max();
+
+  double value;
+
+  for ( int intersectedFaceIndex : intersectedFacesIndices )
+  {
+    QgsMeshFace face = tMesh->triangles().at( intersectedFaceIndex );
+
+    if ( metadata.dataType() == QgsMeshDatasetGroupMetadata::DataType::DataOnFaces || metadata.dataType() == QgsMeshDatasetGroupMetadata::DataType::DataOnVolumes )
+    {
+      value = scalarDatasetValues.at( tMesh->trianglesToNativeFaces().at( intersectedFaceIndex ) );
+      min = std::min( min, value );
+      max = std::max( max, value );
+    }
+    else if ( metadata.dataType() == QgsMeshDatasetGroupMetadata::DataType::DataOnVertices )
+    {
+      QgsMeshVertex vertex;
+
+      for ( int vertexIndex : face )
+      {
+        value = scalarDatasetValues.at( vertexIndex );
+        min = std::min( min, value );
+        max = std::max( max, value );
+      }
+    }
+  }
+
+  return true;
+}
+
 QgsAbstractProfileGenerator *QgsMeshLayer::createProfileGenerator( const QgsProfileRequest &request )
 {
   QGIS_PROTECT_QOBJECT_THREAD_ACCESS
@@ -2210,3 +2331,17 @@ void QgsMeshLayer::setLabeling( QgsAbstractMeshLayerLabeling *labeling )
   mLabeling = labeling;
   triggerRepaint();
 }
+
+bool QgsMeshLayer::datasetsPathUnique( const QString &path )
+{
+  if ( ! mDataProvider )
+  {
+    QgsDebugMsgLevel( QStringLiteral( "Unable to get mesh data provider" ), 2 );
+    return false;
+  }
+
+  if ( mDataProvider->dataSourceUri().contains( path ) )
+    return false;
+
+  return !mExtraDatasetUri.contains( path );
+}
