@@ -16,6 +16,7 @@
  ***************************************************************************/
 
 #include "qgslayoutelevationprofilewidget.h"
+#include "moc_qgslayoutelevationprofilewidget.cpp"
 #include "qgslayoutitemelevationprofile.h"
 #include "qgslayoutitemwidget.h"
 #include "qgslayoutitemregistry.h"
@@ -32,10 +33,11 @@
 #include "qgscurve.h"
 #include "qgslayoutatlas.h"
 #include "qgslayoutreportcontext.h"
+#include "qgsprofilerenderer.h"
 #include "qgsgui.h"
 #include <QMenu>
 
-std::function< void( QgsLayoutElevationProfileWidget *, QMenu * ) > QgsLayoutElevationProfileWidget::sBuildCopyMenuFunction = []( QgsLayoutElevationProfileWidget *, QMenu * ) {};
+std::function<void( QgsLayoutElevationProfileWidget *, QMenu * )> QgsLayoutElevationProfileWidget::sBuildCopyMenuFunction = []( QgsLayoutElevationProfileWidget *, QMenu * ) {};
 
 QgsLayoutElevationProfileWidget::QgsLayoutElevationProfileWidget( QgsLayoutItemElevationProfile *profile )
   : QgsLayoutItemBaseWidget( nullptr, profile )
@@ -49,13 +51,11 @@ QgsLayoutElevationProfileWidget::QgsLayoutElevationProfileWidget( QgsLayoutItemE
   setPanelTitle( tr( "Elevation Profile Properties" ) );
 
   mCopyFromDockMenu = new QMenu( this );
-  connect( mCopyFromDockMenu, &QMenu::aboutToShow, this, [ = ]
-  {
+  connect( mCopyFromDockMenu, &QMenu::aboutToShow, this, [=] {
     sBuildCopyMenuFunction( this, mCopyFromDockMenu );
   } );
 
-  connect( mActionRefresh, &QAction::triggered, this, [ = ]
-  {
+  connect( mActionRefresh, &QAction::triggered, this, [=] {
     if ( !mProfile )
     {
       return;
@@ -81,8 +81,7 @@ QgsLayoutElevationProfileWidget::QgsLayoutElevationProfileWidget( QgsLayoutItemE
   connect( mLayerTree.get(), &QgsLayerTreeGroup::visibilityChanged, this, &QgsLayoutElevationProfileWidget::updateItemLayers );
 
   mSpinTolerance->setClearValue( 0 );
-  connect( mSpinTolerance, qOverload< double >( &QDoubleSpinBox::valueChanged ), this, [ = ]( double value )
-  {
+  connect( mSpinTolerance, qOverload<double>( &QDoubleSpinBox::valueChanged ), this, [=]( double value ) {
     if ( !mProfile || mBlockChanges )
       return;
 
@@ -93,8 +92,7 @@ QgsLayoutElevationProfileWidget::QgsLayoutElevationProfileWidget( QgsLayoutItemE
     mProfile->endCommand();
   } );
 
-  connect( mCheckControlledByAtlas, &QCheckBox::toggled, this, [ = ]
-  {
+  connect( mCheckControlledByAtlas, &QCheckBox::toggled, this, [=] {
     if ( !mProfile || mBlockChanges )
       return;
 
@@ -105,9 +103,36 @@ QgsLayoutElevationProfileWidget::QgsLayoutElevationProfileWidget( QgsLayoutItemE
     mProfile->endCommand();
   } );
 
+  // subsections indicator
+  mSubsectionsSymbolButton->setSymbolType( Qgis::SymbolType::Line );
+  connect( mSubsectionsSymbolButton, &QgsSymbolButton::changed, this, [=] {
+    if ( !mProfile || mBlockChanges )
+      return;
+
+    mProfile->beginCommand( tr( "Change Profile Subsection Indicator" ), QgsLayoutItem::UndoElevationProfileSubsectionLines );
+    mProfile->setSubsectionsSymbol( mSubsectionsSymbolButton->clonedSymbol<QgsLineSymbol>() );
+    mProfile->invalidateCache();
+    mProfile->update();
+    mProfile->endCommand();
+  } );
+  mSubsectionsSymbolButton->setDefaultSymbol( QgsProfilePlotRenderer::defaultSubSectionsSymbol().release() );
+
+  connect( mSubsectionsActivateCheck, &QGroupBox::toggled, this, [=] {
+    if ( !mProfile || mBlockChanges )
+      return;
+
+    const bool subsectionsActivated = mSubsectionsActivateCheck->isChecked();
+    mProfile->beginCommand( tr( "Change Profile Subsection Indicator" ), QgsLayoutItem::UndoElevationProfileSubsectionLines );
+    std::unique_ptr<QgsLineSymbol> subSectionsSymbol( subsectionsActivated ? mSubsectionsSymbolButton->clonedSymbol<QgsLineSymbol>() : nullptr );
+    mProfile->setSubsectionsSymbol( subSectionsSymbol.release() );
+
+    mProfile->invalidateCache();
+    mProfile->update();
+    mProfile->endCommand();
+  } );
+
   mSpinMinDistance->setClearValue( 0 );
-  connect( mSpinMinDistance, qOverload< double >( &QDoubleSpinBox::valueChanged ), this, [ = ]( double value )
-  {
+  connect( mSpinMinDistance, qOverload<double>( &QDoubleSpinBox::valueChanged ), this, [=]( double value ) {
     if ( !mProfile || mBlockChanges )
       return;
 
@@ -118,8 +143,7 @@ QgsLayoutElevationProfileWidget::QgsLayoutElevationProfileWidget( QgsLayoutItemE
     mProfile->endCommand();
   } );
 
-  connect( mSpinMaxDistance, qOverload< double >( &QDoubleSpinBox::valueChanged ), this, [ = ]( double value )
-  {
+  connect( mSpinMaxDistance, qOverload<double>( &QDoubleSpinBox::valueChanged ), this, [=]( double value ) {
     if ( !mProfile || mBlockChanges )
       return;
 
@@ -131,8 +155,7 @@ QgsLayoutElevationProfileWidget::QgsLayoutElevationProfileWidget( QgsLayoutItemE
   } );
 
   mSpinMinElevation->setClearValue( 0 );
-  connect( mSpinMinElevation, qOverload< double >( &QDoubleSpinBox::valueChanged ), this, [ = ]( double value )
-  {
+  connect( mSpinMinElevation, qOverload<double>( &QDoubleSpinBox::valueChanged ), this, [=]( double value ) {
     if ( !mProfile || mBlockChanges )
       return;
 
@@ -143,8 +166,7 @@ QgsLayoutElevationProfileWidget::QgsLayoutElevationProfileWidget( QgsLayoutItemE
     mProfile->endCommand();
   } );
 
-  connect( mSpinMaxElevation, qOverload< double >( &QDoubleSpinBox::valueChanged ), this, [ = ]( double value )
-  {
+  connect( mSpinMaxElevation, qOverload<double>( &QDoubleSpinBox::valueChanged ), this, [=]( double value ) {
     if ( !mProfile || mBlockChanges )
       return;
 
@@ -156,8 +178,7 @@ QgsLayoutElevationProfileWidget::QgsLayoutElevationProfileWidget( QgsLayoutItemE
   } );
 
   mDistanceAxisMajorLinesSymbolButton->setSymbolType( Qgis::SymbolType::Line );
-  connect( mDistanceAxisMajorLinesSymbolButton, &QgsSymbolButton::changed, this, [ = ]
-  {
+  connect( mDistanceAxisMajorLinesSymbolButton, &QgsSymbolButton::changed, this, [=] {
     if ( !mProfile || mBlockChanges )
       return;
 
@@ -170,8 +191,7 @@ QgsLayoutElevationProfileWidget::QgsLayoutElevationProfileWidget( QgsLayoutItemE
   mDistanceAxisMajorLinesSymbolButton->setDefaultSymbol( QgsPlotDefaultSettings::axisGridMajorSymbol() );
 
   mDistanceAxisMinorLinesSymbolButton->setSymbolType( Qgis::SymbolType::Line );
-  connect( mDistanceAxisMinorLinesSymbolButton, &QgsSymbolButton::changed, this, [ = ]
-  {
+  connect( mDistanceAxisMinorLinesSymbolButton, &QgsSymbolButton::changed, this, [=] {
     if ( !mProfile || mBlockChanges )
       return;
 
@@ -183,8 +203,7 @@ QgsLayoutElevationProfileWidget::QgsLayoutElevationProfileWidget( QgsLayoutItemE
   } );
   mDistanceAxisMinorLinesSymbolButton->setDefaultSymbol( QgsPlotDefaultSettings::axisGridMinorSymbol() );
 
-  connect( mDistanceAxisMajorIntervalSpin, qOverload< double >( &QDoubleSpinBox::valueChanged ), this, [ = ]( double value )
-  {
+  connect( mDistanceAxisMajorIntervalSpin, qOverload<double>( &QDoubleSpinBox::valueChanged ), this, [=]( double value ) {
     if ( !mProfile || mBlockChanges )
       return;
 
@@ -195,8 +214,7 @@ QgsLayoutElevationProfileWidget::QgsLayoutElevationProfileWidget( QgsLayoutItemE
     mProfile->endCommand();
   } );
 
-  connect( mDistanceAxisMinorIntervalSpin, qOverload< double >( &QDoubleSpinBox::valueChanged ), this, [ = ]( double value )
-  {
+  connect( mDistanceAxisMinorIntervalSpin, qOverload<double>( &QDoubleSpinBox::valueChanged ), this, [=]( double value ) {
     if ( !mProfile || mBlockChanges )
       return;
 
@@ -207,8 +225,7 @@ QgsLayoutElevationProfileWidget::QgsLayoutElevationProfileWidget( QgsLayoutItemE
     mProfile->endCommand();
   } );
 
-  connect( mDistanceAxisLabelIntervalSpin, qOverload< double >( &QDoubleSpinBox::valueChanged ), this, [ = ]( double value )
-  {
+  connect( mDistanceAxisLabelIntervalSpin, qOverload<double>( &QDoubleSpinBox::valueChanged ), this, [=]( double value ) {
     if ( !mProfile || mBlockChanges )
       return;
 
@@ -220,8 +237,7 @@ QgsLayoutElevationProfileWidget::QgsLayoutElevationProfileWidget( QgsLayoutItemE
   } );
 
   mElevationAxisMajorLinesSymbolButton->setSymbolType( Qgis::SymbolType::Line );
-  connect( mElevationAxisMajorLinesSymbolButton, &QgsSymbolButton::changed, this, [ = ]
-  {
+  connect( mElevationAxisMajorLinesSymbolButton, &QgsSymbolButton::changed, this, [=] {
     if ( !mProfile || mBlockChanges )
       return;
 
@@ -234,8 +250,7 @@ QgsLayoutElevationProfileWidget::QgsLayoutElevationProfileWidget( QgsLayoutItemE
   mElevationAxisMajorLinesSymbolButton->setDefaultSymbol( QgsPlotDefaultSettings::axisGridMajorSymbol() );
 
   mElevationAxisMinorLinesSymbolButton->setSymbolType( Qgis::SymbolType::Line );
-  connect( mElevationAxisMinorLinesSymbolButton, &QgsSymbolButton::changed, this, [ = ]
-  {
+  connect( mElevationAxisMinorLinesSymbolButton, &QgsSymbolButton::changed, this, [=] {
     if ( !mProfile || mBlockChanges )
       return;
 
@@ -247,8 +262,7 @@ QgsLayoutElevationProfileWidget::QgsLayoutElevationProfileWidget( QgsLayoutItemE
   } );
   mElevationAxisMinorLinesSymbolButton->setDefaultSymbol( QgsPlotDefaultSettings::axisGridMinorSymbol() );
 
-  connect( mElevationAxisLabelIntervalSpin, qOverload< double >( &QDoubleSpinBox::valueChanged ), this, [ = ]( double value )
-  {
+  connect( mElevationAxisLabelIntervalSpin, qOverload<double>( &QDoubleSpinBox::valueChanged ), this, [=]( double value ) {
     if ( !mProfile || mBlockChanges )
       return;
 
@@ -259,8 +273,7 @@ QgsLayoutElevationProfileWidget::QgsLayoutElevationProfileWidget( QgsLayoutItemE
     mProfile->endCommand();
   } );
 
-  connect( mElevationAxisMajorIntervalSpin, qOverload< double >( &QDoubleSpinBox::valueChanged ), this, [ = ]( double value )
-  {
+  connect( mElevationAxisMajorIntervalSpin, qOverload<double>( &QDoubleSpinBox::valueChanged ), this, [=]( double value ) {
     if ( !mProfile || mBlockChanges )
       return;
 
@@ -271,8 +284,7 @@ QgsLayoutElevationProfileWidget::QgsLayoutElevationProfileWidget( QgsLayoutItemE
     mProfile->endCommand();
   } );
 
-  connect( mElevationAxisMinorIntervalSpin, qOverload< double >( &QDoubleSpinBox::valueChanged ), this, [ = ]( double value )
-  {
+  connect( mElevationAxisMinorIntervalSpin, qOverload<double>( &QDoubleSpinBox::valueChanged ), this, [=]( double value ) {
     if ( !mProfile || mBlockChanges )
       return;
 
@@ -284,8 +296,7 @@ QgsLayoutElevationProfileWidget::QgsLayoutElevationProfileWidget( QgsLayoutItemE
   } );
 
   mChartBackgroundSymbolButton->setSymbolType( Qgis::SymbolType::Fill );
-  connect( mChartBackgroundSymbolButton, &QgsSymbolButton::changed, this, [ = ]
-  {
+  connect( mChartBackgroundSymbolButton, &QgsSymbolButton::changed, this, [=] {
     if ( !mProfile || mBlockChanges )
       return;
 
@@ -298,8 +309,7 @@ QgsLayoutElevationProfileWidget::QgsLayoutElevationProfileWidget( QgsLayoutItemE
   mChartBackgroundSymbolButton->setDefaultSymbol( QgsPlotDefaultSettings::chartBackgroundSymbol() );
 
   mChartBorderSymbolButton->setSymbolType( Qgis::SymbolType::Fill );
-  connect( mChartBorderSymbolButton, &QgsSymbolButton::changed, this, [ = ]
-  {
+  connect( mChartBorderSymbolButton, &QgsSymbolButton::changed, this, [=] {
     if ( !mProfile || mBlockChanges )
       return;
 
@@ -311,16 +321,14 @@ QgsLayoutElevationProfileWidget::QgsLayoutElevationProfileWidget( QgsLayoutItemE
   } );
   mChartBorderSymbolButton->setDefaultSymbol( QgsPlotDefaultSettings::chartBorderSymbol() );
 
-  connect( mDistanceAxisLabelFormatButton, &QPushButton::clicked, this, [ = ]
-  {
+  connect( mDistanceAxisLabelFormatButton, &QPushButton::clicked, this, [=] {
     if ( !mProfile || mBlockChanges )
       return;
 
     QgsNumericFormatSelectorWidget *widget = new QgsNumericFormatSelectorWidget( this );
     widget->setPanelTitle( tr( "Distance Number Format" ) );
     widget->setFormat( mProfile->plot()->xAxis().numericFormat() );
-    connect( widget, &QgsNumericFormatSelectorWidget::changed, this, [ = ]
-    {
+    connect( widget, &QgsNumericFormatSelectorWidget::changed, this, [=] {
       mProfile->beginCommand( tr( "Change Profile Chart Distance Format" ), QgsLayoutItem::UndoElevationProfileDistanceFormat );
       mProfile->plot()->xAxis().setNumericFormat( widget->format() );
       mProfile->invalidateCache();
@@ -330,16 +338,14 @@ QgsLayoutElevationProfileWidget::QgsLayoutElevationProfileWidget( QgsLayoutItemE
     openPanel( widget );
   } );
 
-  connect( mElevationAxisLabelFormatButton, &QPushButton::clicked, this, [ = ]
-  {
+  connect( mElevationAxisLabelFormatButton, &QPushButton::clicked, this, [=] {
     if ( !mProfile || mBlockChanges )
       return;
 
     QgsNumericFormatSelectorWidget *widget = new QgsNumericFormatSelectorWidget( this );
     widget->setPanelTitle( tr( "Elevation Number Format" ) );
     widget->setFormat( mProfile->plot()->yAxis().numericFormat() );
-    connect( widget, &QgsNumericFormatSelectorWidget::changed, this, [ = ]
-    {
+    connect( widget, &QgsNumericFormatSelectorWidget::changed, this, [=] {
       mProfile->beginCommand( tr( "Change Profile Chart Elevation Format" ), QgsLayoutItem::UndoElevationProfileElevationFormat );
       mProfile->plot()->yAxis().setNumericFormat( widget->format() );
       mProfile->invalidateCache();
@@ -354,8 +360,7 @@ QgsLayoutElevationProfileWidget::QgsLayoutElevationProfileWidget( QgsLayoutItemE
   mDistanceAxisLabelFontButton->setMode( QgsFontButton::ModeTextRenderer );
   mElevationAxisLabelFontButton->setMode( QgsFontButton::ModeTextRenderer );
 
-  connect( mDistanceAxisLabelFontButton, &QgsFontButton::changed, this, [ = ]
-  {
+  connect( mDistanceAxisLabelFontButton, &QgsFontButton::changed, this, [=] {
     if ( !mProfile || mBlockChanges )
       return;
 
@@ -366,8 +371,7 @@ QgsLayoutElevationProfileWidget::QgsLayoutElevationProfileWidget( QgsLayoutItemE
     mProfile->update();
   } );
 
-  connect( mElevationAxisLabelFontButton, &QgsFontButton::changed, this, [ = ]
-  {
+  connect( mElevationAxisLabelFontButton, &QgsFontButton::changed, this, [=] {
     if ( !mProfile || mBlockChanges )
       return;
 
@@ -379,8 +383,7 @@ QgsLayoutElevationProfileWidget::QgsLayoutElevationProfileWidget( QgsLayoutItemE
   } );
 
   mSpinLeftMargin->setClearValue( 0 );
-  connect( mSpinLeftMargin, qOverload< double >( &QDoubleSpinBox::valueChanged ), this, [ = ]( double value )
-  {
+  connect( mSpinLeftMargin, qOverload<double>( &QDoubleSpinBox::valueChanged ), this, [=]( double value ) {
     if ( !mProfile || mBlockChanges )
       return;
 
@@ -394,8 +397,7 @@ QgsLayoutElevationProfileWidget::QgsLayoutElevationProfileWidget( QgsLayoutItemE
   } );
 
   mSpinRightMargin->setClearValue( 0 );
-  connect( mSpinRightMargin, qOverload< double >( &QDoubleSpinBox::valueChanged ), this, [ = ]( double value )
-  {
+  connect( mSpinRightMargin, qOverload<double>( &QDoubleSpinBox::valueChanged ), this, [=]( double value ) {
     if ( !mProfile || mBlockChanges )
       return;
 
@@ -409,8 +411,7 @@ QgsLayoutElevationProfileWidget::QgsLayoutElevationProfileWidget( QgsLayoutItemE
   } );
 
   mSpinTopMargin->setClearValue( 0 );
-  connect( mSpinTopMargin, qOverload< double >( &QDoubleSpinBox::valueChanged ), this, [ = ]( double value )
-  {
+  connect( mSpinTopMargin, qOverload<double>( &QDoubleSpinBox::valueChanged ), this, [=]( double value ) {
     if ( !mProfile || mBlockChanges )
       return;
 
@@ -424,8 +425,7 @@ QgsLayoutElevationProfileWidget::QgsLayoutElevationProfileWidget( QgsLayoutItemE
   } );
 
   mSpinBottomMargin->setClearValue( 0 );
-  connect( mSpinBottomMargin, qOverload< double >( &QDoubleSpinBox::valueChanged ), this, [ = ]( double value )
-  {
+  connect( mSpinBottomMargin, qOverload<double>( &QDoubleSpinBox::valueChanged ), this, [=]( double value ) {
     if ( !mProfile || mBlockChanges )
       return;
 
@@ -464,13 +464,12 @@ QgsLayoutElevationProfileWidget::QgsLayoutElevationProfileWidget( QgsLayoutItemE
     mDistanceUnitCombo->addItem( title, QVariant::fromValue( unit ) );
   }
 
-  connect( mDistanceUnitCombo, qOverload< int >( &QComboBox::currentIndexChanged ), this, [ = ]( int )
-  {
+  connect( mDistanceUnitCombo, qOverload<int>( &QComboBox::currentIndexChanged ), this, [=]( int ) {
     if ( !mProfile || mBlockChanges )
       return;
 
     mProfile->beginCommand( tr( "Change Profile Chart Units" ) );
-    mProfile->setDistanceUnit( mDistanceUnitCombo->currentData().value< Qgis::DistanceUnit >() );
+    mProfile->setDistanceUnit( mDistanceUnitCombo->currentData().value<Qgis::DistanceUnit>() );
     mProfile->invalidateCache();
     mProfile->update();
     mProfile->endCommand();
@@ -481,13 +480,12 @@ QgsLayoutElevationProfileWidget::QgsLayoutElevationProfileWidget( QgsLayoutItemE
   mDistanceLabelsCombo->addItem( tr( "First Value" ), QVariant::fromValue( Qgis::PlotAxisSuffixPlacement::FirstLabel ) );
   mDistanceLabelsCombo->addItem( tr( "Last Value" ), QVariant::fromValue( Qgis::PlotAxisSuffixPlacement::LastLabel ) );
   mDistanceLabelsCombo->addItem( tr( "First and Last Values" ), QVariant::fromValue( Qgis::PlotAxisSuffixPlacement::FirstAndLastLabels ) );
-  connect( mDistanceLabelsCombo, qOverload< int >( &QComboBox::currentIndexChanged ), this, [ = ]( int )
-  {
+  connect( mDistanceLabelsCombo, qOverload<int>( &QComboBox::currentIndexChanged ), this, [=]( int ) {
     if ( !mProfile || mBlockChanges )
       return;
 
     mProfile->beginCommand( tr( "Change Profile Chart Label Placement" ) );
-    mProfile->plot()->xAxis().setLabelSuffixPlacement( mDistanceLabelsCombo->currentData().value< Qgis::PlotAxisSuffixPlacement >() );
+    mProfile->plot()->xAxis().setLabelSuffixPlacement( mDistanceLabelsCombo->currentData().value<Qgis::PlotAxisSuffixPlacement>() );
     mProfile->invalidateCache();
     mProfile->update();
     mProfile->endCommand();
@@ -523,6 +521,7 @@ QgsLayoutElevationProfileWidget::QgsLayoutElevationProfileWidget( QgsLayoutItemE
 
   setGuiElementValues();
 
+  mSubsectionsSymbolButton->registerExpressionContextGenerator( mProfile );
   mDistanceAxisMajorLinesSymbolButton->registerExpressionContextGenerator( mProfile );
   mDistanceAxisMinorLinesSymbolButton->registerExpressionContextGenerator( mProfile );
   mElevationAxisMajorLinesSymbolButton->registerExpressionContextGenerator( mProfile );
@@ -532,6 +531,7 @@ QgsLayoutElevationProfileWidget::QgsLayoutElevationProfileWidget( QgsLayoutItemE
   mDistanceAxisLabelFontButton->registerExpressionContextGenerator( mProfile );
   mElevationAxisLabelFontButton->registerExpressionContextGenerator( mProfile );
 
+  mSubsectionsSymbolButton->setLayer( coverageLayer() );
   mDistanceAxisMajorLinesSymbolButton->setLayer( coverageLayer() );
   mDistanceAxisMinorLinesSymbolButton->setLayer( coverageLayer() );
   mElevationAxisMajorLinesSymbolButton->setLayer( coverageLayer() );
@@ -543,8 +543,8 @@ QgsLayoutElevationProfileWidget::QgsLayoutElevationProfileWidget( QgsLayoutItemE
 
   if ( mProfile->layout() )
   {
-    connect( &mProfile->layout()->reportContext(), &QgsLayoutReportContext::layerChanged, this, [ = ]( QgsVectorLayer * layer )
-    {
+    connect( &mProfile->layout()->reportContext(), &QgsLayoutReportContext::layerChanged, this, [=]( QgsVectorLayer *layer ) {
+      mSubsectionsSymbolButton->setLayer( layer );
       mDistanceAxisMajorLinesSymbolButton->setLayer( layer );
       mDistanceAxisMinorLinesSymbolButton->setLayer( layer );
       mElevationAxisMajorLinesSymbolButton->setLayer( layer );
@@ -555,8 +555,7 @@ QgsLayoutElevationProfileWidget::QgsLayoutElevationProfileWidget( QgsLayoutItemE
       mChartBorderSymbolButton->setLayer( layer );
     } );
 
-    connect( &mProfile->layout()->reportContext(), &QgsLayoutReportContext::layerChanged,
-             this, &QgsLayoutElevationProfileWidget::atlasLayerChanged );
+    connect( &mProfile->layout()->reportContext(), &QgsLayoutReportContext::layerChanged, this, &QgsLayoutElevationProfileWidget::atlasLayerChanged );
   }
 
   if ( QgsLayoutAtlas *atlas = layoutAtlas() )
@@ -648,6 +647,15 @@ void QgsLayoutElevationProfileWidget::copySettingsFromProfileCanvas( QgsElevatio
   mElevationAxisLabelIntervalSpin->setClearValue( canvas->plot().yAxis().labelInterval() );
   mProfile->plot()->yAxis().setLabelInterval( canvas->plot().yAxis().labelInterval() );
 
+  const QgsLineSymbol *subSectionsSymbol = canvas->subsectionsSymbol() ? canvas->subsectionsSymbol() : nullptr;
+  const bool subSectionsEnabled = static_cast< bool >( subSectionsSymbol );
+  mSubsectionsActivateCheck->setChecked( subSectionsEnabled );
+  if ( subSectionsSymbol )
+  {
+    mSubsectionsSymbolButton->setSymbol( subSectionsSymbol->clone() );
+    mProfile->setSubsectionsSymbol( subSectionsSymbol->clone() );
+  }
+
   QList<QgsMapLayer *> canvasLayers = canvas->layers();
   // canvas layers are in opposite direction to what the layout item requires
   std::reverse( canvasLayers.begin(), canvasLayers.end() );
@@ -674,12 +682,13 @@ bool QgsLayoutElevationProfileWidget::setNewItem( QgsLayoutItem *item )
     disconnect( mProfile, &QgsLayoutObject::changed, this, &QgsLayoutElevationProfileWidget::setGuiElementValues );
   }
 
-  mProfile = qobject_cast< QgsLayoutItemElevationProfile * >( item );
+  mProfile = qobject_cast<QgsLayoutItemElevationProfile *>( item );
   mItemPropertiesWidget->setItem( mProfile );
 
   if ( mProfile )
   {
     connect( mProfile, &QgsLayoutObject::changed, this, &QgsLayoutElevationProfileWidget::setGuiElementValues );
+    mSubsectionsSymbolButton->registerExpressionContextGenerator( mProfile );
     mDistanceAxisMajorLinesSymbolButton->registerExpressionContextGenerator( mProfile );
     mDistanceAxisMinorLinesSymbolButton->registerExpressionContextGenerator( mProfile );
     mElevationAxisMajorLinesSymbolButton->registerExpressionContextGenerator( mProfile );
@@ -707,13 +716,19 @@ void QgsLayoutElevationProfileWidget::setGuiElementValues()
   mSpinMinElevation->setValue( mProfile->plot()->yMinimum() );
   mSpinMaxElevation->setValue( mProfile->plot()->yMaximum() );
 
+  mSubsectionsActivateCheck->setChecked( mProfile->subsectionsSymbol() );
+  if ( mProfile->subsectionsSymbol() )
+  {
+    mSubsectionsSymbolButton->setSymbol( mProfile->subsectionsSymbol()->clone() );
+  }
+
   if ( mProfile->plot()->xAxis().gridMajorSymbol() )
     mDistanceAxisMajorLinesSymbolButton->setSymbol( mProfile->plot()->xAxis().gridMajorSymbol()->clone() );
   if ( mProfile->plot()->xAxis().gridMinorSymbol() )
     mDistanceAxisMinorLinesSymbolButton->setSymbol( mProfile->plot()->xAxis().gridMinorSymbol()->clone() );
   if ( mProfile->plot()->yAxis().gridMajorSymbol() )
     mElevationAxisMajorLinesSymbolButton->setSymbol( mProfile->plot()->yAxis().gridMajorSymbol()->clone() );
-  if ( mProfile->plot()->yAxis().gridMajorSymbol() )
+  if ( mProfile->plot()->yAxis().gridMinorSymbol() )
     mElevationAxisMinorLinesSymbolButton->setSymbol( mProfile->plot()->yAxis().gridMinorSymbol()->clone() );
 
   mDistanceAxisLabelFontButton->setTextFormat( mProfile->plot()->xAxis().textFormat() );
@@ -740,7 +755,7 @@ void QgsLayoutElevationProfileWidget::setGuiElementValues()
   mSpinTopMargin->setValue( mProfile->plot()->margins().top() );
   mSpinBottomMargin->setValue( mProfile->plot()->margins().bottom() );
 
-  const QList<QgsLayerTreeLayer *>  layers = mLayerTree->findLayers();
+  const QList<QgsLayerTreeLayer *> layers = mLayerTree->findLayers();
   for ( QgsLayerTreeLayer *layer : layers )
   {
     layer->setItemVisibilityChecked( mProfile->layers().contains( layer->layer() ) );
@@ -772,7 +787,7 @@ void QgsLayoutElevationProfileWidget::updateItemLayers()
     return;
 
   QList<QgsMapLayer *> layers;
-  const QList< QgsMapLayer * > layerOrder = mLayerTree->layerOrder();
+  const QList<QgsMapLayer *> layerOrder = mLayerTree->layerOrder();
   layers.reserve( layerOrder.size() );
   for ( QgsMapLayer *layer : layerOrder )
   {
@@ -786,8 +801,7 @@ void QgsLayoutElevationProfileWidget::updateItemLayers()
 
 void QgsLayoutElevationProfileWidget::layoutAtlasToggled( bool atlasEnabled )
 {
-  if ( atlasEnabled &&
-       mProfile && mProfile->layout() && mProfile->layout()->reportContext().layer()
+  if ( atlasEnabled && mProfile && mProfile->layout() && mProfile->layout()->reportContext().layer()
        && mProfile->layout()->reportContext().layer()->geometryType() == Qgis::GeometryType::Line )
   {
     mCheckControlledByAtlas->setEnabled( true );
