@@ -73,7 +73,7 @@ QgsLayoutItemMap *findSensibleDefaultLinkedMapItem( QgsLayoutItem *referenceItem
   }
 
   // nope, no selection... hm, was the item drawn over a map? If so, use the topmost intersecting one
-  double largestZValue = std::numeric_limits< double >::lowest();
+  double largestZValue = std::numeric_limits<double>::lowest();
   for ( QgsLayoutItemMap *map : std::as_const( mapItems ) )
   {
     if ( map->collidesWithItem( referenceItem ) && map->zValue() > largestZValue )
@@ -96,16 +96,13 @@ void QgsLayoutGuiUtils::registerGuiForKnownItemTypes( QgsMapCanvas *mapCanvas )
   registry->addItemGroup( QgsLayoutItemGuiGroup( QStringLiteral( "shapes" ), QObject::tr( "Shape" ), QgsApplication::getThemeIcon( QStringLiteral( "/mActionAddBasicShape.svg" ) ) ) );
   registry->addItemGroup( QgsLayoutItemGuiGroup( QStringLiteral( "nodes" ), QObject::tr( "Node Item" ), QgsApplication::getThemeIcon( QStringLiteral( "/mActionAddNodesItem.svg" ) ) ) );
 
-  auto createRubberBand = ( []( QgsLayoutView * view )->QgsLayoutViewRubberBand *
-  {
+  auto createRubberBand = ( []( QgsLayoutView *view ) -> QgsLayoutViewRubberBand * {
     return new QgsLayoutViewRectangularRubberBand( view );
   } );
-  auto createEllipseBand = ( []( QgsLayoutView * view )->QgsLayoutViewRubberBand *
-  {
+  auto createEllipseBand = ( []( QgsLayoutView *view ) -> QgsLayoutViewRubberBand * {
     return new QgsLayoutViewEllipticalRubberBand( view );
   } );
-  auto createTriangleBand = ( []( QgsLayoutView * view )->QgsLayoutViewRubberBand *
-  {
+  auto createTriangleBand = ( []( QgsLayoutView *view ) -> QgsLayoutViewRubberBand * {
     return new QgsLayoutViewTriangleRubberBand( view );
   } );
 
@@ -115,14 +112,9 @@ void QgsLayoutGuiUtils::registerGuiForKnownItemTypes( QgsMapCanvas *mapCanvas )
 
   // map item
 
-  auto mapItemMetadata = std::make_unique< QgsLayoutItemGuiMetadata >( QgsLayoutItemRegistry::LayoutMap, QObject::tr( "Map" ), QgsApplication::getThemeIcon( QStringLiteral( "/mActionAddMap.svg" ) ),
-                         [ = ]( QgsLayoutItem * item )->QgsLayoutItemBaseWidget *
-  {
-    return new QgsLayoutMapWidget( qobject_cast< QgsLayoutItemMap * >( item ), mapCanvas );
-  }, createRubberBand );
-  mapItemMetadata->setItemAddedToLayoutFunction( [ = ]( QgsLayoutItem * item, const QVariantMap & )
-  {
-    QgsLayoutItemMap *map = qobject_cast< QgsLayoutItemMap * >( item );
+  auto mapItemMetadata = std::make_unique<QgsLayoutItemGuiMetadata>( QgsLayoutItemRegistry::LayoutMap, QObject::tr( "Map" ), QgsApplication::getThemeIcon( QStringLiteral( "/mActionAddMap.svg" ) ), [=]( QgsLayoutItem *item ) -> QgsLayoutItemBaseWidget * { return new QgsLayoutMapWidget( qobject_cast<QgsLayoutItemMap *>( item ), mapCanvas ); }, createRubberBand );
+  mapItemMetadata->setItemAddedToLayoutFunction( [=]( QgsLayoutItem *item, const QVariantMap & ) {
+    QgsLayoutItemMap *map = qobject_cast<QgsLayoutItemMap *>( item );
     Q_ASSERT( map );
 
     //get the color for map canvas background and set map background color accordingly
@@ -166,23 +158,14 @@ void QgsLayoutGuiUtils::registerGuiForKnownItemTypes( QgsMapCanvas *mapCanvas )
 
   // picture item
 
-  registry->addLayoutItemGuiMetadata( new QgsLayoutItemGuiMetadata( QgsLayoutItemRegistry::LayoutPicture, QObject::tr( "Picture" ), QgsApplication::getThemeIcon( QStringLiteral( "/mActionAddImage.svg" ) ),
-                                      [ = ]( QgsLayoutItem * item )->QgsLayoutItemBaseWidget *
-  {
-    return new QgsLayoutPictureWidget( qobject_cast< QgsLayoutItemPicture * >( item ) );
-  }, createRubberBand ) );
+  registry->addLayoutItemGuiMetadata( new QgsLayoutItemGuiMetadata( QgsLayoutItemRegistry::LayoutPicture, QObject::tr( "Picture" ), QgsApplication::getThemeIcon( QStringLiteral( "/mActionAddImage.svg" ) ), [=]( QgsLayoutItem *item ) -> QgsLayoutItemBaseWidget * { return new QgsLayoutPictureWidget( qobject_cast<QgsLayoutItemPicture *>( item ) ); }, createRubberBand ) );
 
 
   // label item
 
-  auto labelItemMetadata = std::make_unique< QgsLayoutItemGuiMetadata >( QgsLayoutItemRegistry::LayoutLabel, QObject::tr( "Label" ), QgsApplication::getThemeIcon( QStringLiteral( "/mActionLabel.svg" ) ),
-                           [ = ]( QgsLayoutItem * item )->QgsLayoutItemBaseWidget *
-  {
-    return new QgsLayoutLabelWidget( qobject_cast< QgsLayoutItemLabel * >( item ) );
-  }, createRubberBand );
-  labelItemMetadata->setItemAddedToLayoutFunction( [ = ]( QgsLayoutItem * item, const QVariantMap & properties )
-  {
-    QgsLayoutItemLabel *label = qobject_cast< QgsLayoutItemLabel * >( item );
+  auto labelItemMetadata = std::make_unique<QgsLayoutItemGuiMetadata>( QgsLayoutItemRegistry::LayoutLabel, QObject::tr( "Label" ), QgsApplication::getThemeIcon( QStringLiteral( "/mActionLabel.svg" ) ), [=]( QgsLayoutItem *item ) -> QgsLayoutItemBaseWidget * { return new QgsLayoutLabelWidget( qobject_cast<QgsLayoutItemLabel *>( item ) ); }, createRubberBand );
+  labelItemMetadata->setItemAddedToLayoutFunction( [=]( QgsLayoutItem *item, const QVariantMap &properties ) {
+    QgsLayoutItemLabel *label = qobject_cast<QgsLayoutItemLabel *>( item );
     Q_ASSERT( label );
 
     label->setText( properties.value( QStringLiteral( "expression" ) ).toString().isEmpty() ? QObject::tr( "Lorem ipsum" ) : QStringLiteral( "[% %1 %]" ).arg( properties.value( QStringLiteral( "expression" ) ).toString() ) );
@@ -199,19 +182,68 @@ void QgsLayoutGuiUtils::registerGuiForKnownItemTypes( QgsMapCanvas *mapCanvas )
     label->attemptSetSceneRect( QRectF( label->pos().x(), label->pos().y(), labelWidth, labelHeight ) );
   } );
 
+  labelItemMetadata->setItemDoubleClickedFunction( [=]( QgsLayoutItem *item, Qgis::MouseHandlesAction action ) {
+    QgsLayoutItemLabel *label = qobject_cast<QgsLayoutItemLabel *>( item );
+
+    // size to text doesn't have any real meaning for HTML content, skip it
+    if ( label->mode() == QgsLayoutItemLabel::ModeHtml )
+      return;
+
+    Q_ASSERT( label );
+    QgsLayoutItem::ReferencePoint reference = QgsLayoutItem::ReferencePoint::UpperLeft;
+    switch ( action )
+    {
+      case Qgis::MouseHandlesAction::MoveItem:
+      case Qgis::MouseHandlesAction::NoAction:
+      case Qgis::MouseHandlesAction::SelectItem:
+        return;
+
+      case Qgis::MouseHandlesAction::ResizeUp:
+        reference = QgsLayoutItem::ReferencePoint::LowerMiddle;
+        break;
+
+      case Qgis::MouseHandlesAction::ResizeDown:
+        reference = QgsLayoutItem::ReferencePoint::UpperMiddle;
+        break;
+
+      case Qgis::MouseHandlesAction::ResizeLeft:
+        reference = QgsLayoutItem::ReferencePoint::MiddleRight;
+        break;
+
+      case Qgis::MouseHandlesAction::ResizeRight:
+        reference = QgsLayoutItem::ReferencePoint::MiddleLeft;
+        break;
+
+      case Qgis::MouseHandlesAction::ResizeLeftUp:
+        reference = QgsLayoutItem::ReferencePoint::LowerRight;
+        break;
+
+      case Qgis::MouseHandlesAction::ResizeRightUp:
+        reference = QgsLayoutItem::ReferencePoint::LowerLeft;
+        break;
+
+      case Qgis::MouseHandlesAction::ResizeLeftDown:
+        reference = QgsLayoutItem::ReferencePoint::UpperRight;
+        break;
+
+      case Qgis::MouseHandlesAction::ResizeRightDown:
+        reference = QgsLayoutItem::ReferencePoint::UpperLeft;
+        break;
+    }
+
+    label->beginCommand( QObject::tr( "Resize to Text" ) );
+    label->adjustSizeToText( reference );
+    label->endCommand();
+  } );
+
   registry->addLayoutItemGuiMetadata( labelItemMetadata.release() );
 
 
   // legend item
 
-  auto legendItemMetadata = std::make_unique< QgsLayoutItemGuiMetadata >( QgsLayoutItemRegistry::LayoutLegend, QObject::tr( "Legend" ), QgsApplication::getThemeIcon( QStringLiteral( "/mActionAddLegend.svg" ) ),
-                            [ = ]( QgsLayoutItem * item )->QgsLayoutItemBaseWidget *
-  {
-    return new QgsLayoutLegendWidget( qobject_cast< QgsLayoutItemLegend * >( item ), mapCanvas );
-  }, createRubberBand );
-  legendItemMetadata->setItemAddedToLayoutFunction( [ = ]( QgsLayoutItem * item, const QVariantMap & )
-  {
-    QgsLayoutItemLegend *legend = qobject_cast< QgsLayoutItemLegend * >( item );
+  auto legendItemMetadata = std::make_unique<QgsLayoutItemGuiMetadata>( QgsLayoutItemRegistry::LayoutLegend, QObject::tr( "Legend" ), QgsApplication::getThemeIcon( QStringLiteral( "/mActionAddLegend.svg" ) ), [=]( QgsLayoutItem *item ) -> QgsLayoutItemBaseWidget * { return new QgsLayoutLegendWidget( qobject_cast<QgsLayoutItemLegend *>( item ), mapCanvas ); }, createRubberBand );
+  legendItemMetadata->setItemAddedToLayoutFunction( [=]( QgsLayoutItem *item, const QVariantMap & ) {
+    QgsLayoutItemLegend *legend = qobject_cast<QgsLayoutItemLegend *>( item );
     Q_ASSERT( legend );
 
     // try to find a good map to link the legend with by default
@@ -221,9 +253,9 @@ void QgsLayoutGuiUtils::registerGuiForKnownItemTypes( QgsMapCanvas *mapCanvas )
     {
       // for right-to-left locales, use an appropriate default layout
       legend->setSymbolAlignment( Qt::AlignRight );
-      legend->rstyle( QgsLegendStyle::Group ).setAlignment( Qt::AlignRight );
-      legend->rstyle( QgsLegendStyle::Subgroup ).setAlignment( Qt::AlignRight );
-      legend->rstyle( QgsLegendStyle::SymbolLabel ).setAlignment( Qt::AlignRight );
+      legend->rstyle( Qgis::LegendComponent::Group ).setAlignment( Qt::AlignRight );
+      legend->rstyle( Qgis::LegendComponent::Subgroup ).setAlignment( Qt::AlignRight );
+      legend->rstyle( Qgis::LegendComponent::SymbolLabel ).setAlignment( Qt::AlignRight );
       legend->setTitleAlignment( Qt::AlignRight );
     }
 
@@ -235,21 +267,21 @@ void QgsLayoutGuiUtils::registerGuiForKnownItemTypes( QgsMapCanvas *mapCanvas )
       QFont font;
       QgsFontUtils::setFontFamily( font, defaultFontString );
 
-      QgsTextFormat f = legend->rstyle( QgsLegendStyle::Title ).textFormat();
+      QgsTextFormat f = legend->rstyle( Qgis::LegendComponent::Title ).textFormat();
       f.setFont( font );
-      legend->rstyle( QgsLegendStyle::Title ).setTextFormat( f );
+      legend->rstyle( Qgis::LegendComponent::Title ).setTextFormat( f );
 
-      f = legend->rstyle( QgsLegendStyle::Group ).textFormat();
+      f = legend->rstyle( Qgis::LegendComponent::Group ).textFormat();
       f.setFont( font );
-      legend->rstyle( QgsLegendStyle::Group ).setTextFormat( f );
+      legend->rstyle( Qgis::LegendComponent::Group ).setTextFormat( f );
 
-      f = legend->rstyle( QgsLegendStyle::Subgroup ).textFormat();
+      f = legend->rstyle( Qgis::LegendComponent::Subgroup ).textFormat();
       f.setFont( font );
-      legend->rstyle( QgsLegendStyle::Subgroup ).setTextFormat( f );
+      legend->rstyle( Qgis::LegendComponent::Subgroup ).setTextFormat( f );
 
-      f = legend->rstyle( QgsLegendStyle::SymbolLabel ).textFormat();
+      f = legend->rstyle( Qgis::LegendComponent::SymbolLabel ).textFormat();
       f.setFont( font );
-      legend->rstyle( QgsLegendStyle::SymbolLabel ).setTextFormat( f );
+      legend->rstyle( Qgis::LegendComponent::SymbolLabel ).setTextFormat( f );
     }
 
     legend->updateLegend();
@@ -259,16 +291,14 @@ void QgsLayoutGuiUtils::registerGuiForKnownItemTypes( QgsMapCanvas *mapCanvas )
 
   // scalebar item
 
-  auto scalebarItemMetadata = std::make_unique< QgsLayoutItemGuiMetadata >( QgsLayoutItemRegistry::LayoutScaleBar, QObject::tr( "Scale Bar" ), QgsApplication::getThemeIcon( QStringLiteral( "/mActionScaleBar.svg" ) ),
-                              [ = ]( QgsLayoutItem * item )->QgsLayoutItemBaseWidget *
-  {
-    return new QgsLayoutScaleBarWidget( qobject_cast< QgsLayoutItemScaleBar * >( item ) );
-  }, createRubberBand );
-  scalebarItemMetadata->setItemAddedToLayoutFunction( [ = ]( QgsLayoutItem * item, const QVariantMap & )
-  {
-    QgsLayoutItemScaleBar *scalebar = qobject_cast< QgsLayoutItemScaleBar * >( item );
+  auto scalebarItemMetadata = std::make_unique<QgsLayoutItemGuiMetadata>( QgsLayoutItemRegistry::LayoutScaleBar, QObject::tr( "Scale Bar" ), QgsApplication::getThemeIcon( QStringLiteral( "/mActionScaleBar.svg" ) ), [=]( QgsLayoutItem *item ) -> QgsLayoutItemBaseWidget * { return new QgsLayoutScaleBarWidget( qobject_cast<QgsLayoutItemScaleBar *>( item ) ); }, createRubberBand );
+  scalebarItemMetadata->setItemAddedToLayoutFunction( [=]( QgsLayoutItem *item, const QVariantMap & ) {
+    QgsLayoutItemScaleBar *scalebar = qobject_cast<QgsLayoutItemScaleBar *>( item );
     Q_ASSERT( scalebar );
 
+    // default to project's scale calculation method
+    scalebar->setMethod( scalebar->layout()->project()->scaleMethod() );
+
     // try to find a good map to link the scalebar with by default
     if ( QgsLayoutItemMap *targetMap = findSensibleDefaultLinkedMapItem( scalebar ) )
     {
@@ -281,17 +311,16 @@ void QgsLayoutGuiUtils::registerGuiForKnownItemTypes( QgsMapCanvas *mapCanvas )
 
 
   // north arrow
-  std::unique_ptr< QgsLayoutItemGuiMetadata > northArrowMetadata = std::make_unique< QgsLayoutItemGuiMetadata>(
-        QgsLayoutItemRegistry::LayoutPicture, QObject::tr( "North Arrow" ), QgsApplication::getThemeIcon( QStringLiteral( "/north_arrow.svg" ) ),
-        [ = ]( QgsLayoutItem * item )->QgsLayoutItemBaseWidget *
-  {
-    return new QgsLayoutPictureWidget( qobject_cast< QgsLayoutItemPicture * >( item ) );
-  }, createRubberBand );
-  northArrowMetadata->setItemCreationFunction( []( QgsLayout * layout )->QgsLayoutItem *
-  {
-
+  auto northArrowMetadata = std::make_unique<QgsLayoutItemGuiMetadata>(
+    QgsLayoutItemRegistry::LayoutPicture, QObject::tr( "North Arrow" ), QgsApplication::getThemeIcon( QStringLiteral( "/north_arrow.svg" ) ),
+    [=]( QgsLayoutItem *item ) -> QgsLayoutItemBaseWidget * {
+      return new QgsLayoutPictureWidget( qobject_cast<QgsLayoutItemPicture *>( item ) );
+    },
+    createRubberBand
+  );
+  northArrowMetadata->setItemCreationFunction( []( QgsLayout *layout ) -> QgsLayoutItem * {
     // count how many existing north arrows are already in layout
-    QList< QgsLayoutItemPicture * > pictureItems;
+    QList<QgsLayoutItemPicture *> pictureItems;
     layout->layoutItems( pictureItems );
     int northArrowCount = 0;
 
@@ -305,16 +334,15 @@ void QgsLayoutGuiUtils::registerGuiForKnownItemTypes( QgsMapCanvas *mapCanvas )
         northArrowCount++;
     }
 
-    std::unique_ptr< QgsLayoutItemPicture > picture = std::make_unique< QgsLayoutItemPicture >( layout );
+    auto picture = std::make_unique<QgsLayoutItemPicture>( layout );
     picture->setNorthMode( QgsLayoutItemPicture::GridNorth );
     picture->setPicturePath( defaultPath );
     // set an id by default, so that north arrows are discernible in layout item lists
     picture->setId( northArrowCount > 0 ? QObject::tr( "North Arrow %1" ).arg( northArrowCount + 1 ) : QObject::tr( "North Arrow" ) );
     return picture.release();
   } );
-  northArrowMetadata->setItemAddedToLayoutFunction( [ = ]( QgsLayoutItem * item, const QVariantMap & )
-  {
-    QgsLayoutItemPicture *picture = qobject_cast< QgsLayoutItemPicture * >( item );
+  northArrowMetadata->setItemAddedToLayoutFunction( [=]( QgsLayoutItem *item, const QVariantMap & ) {
+    QgsLayoutItemPicture *picture = qobject_cast<QgsLayoutItemPicture *>( item );
     Q_ASSERT( picture );
 
     QList<QgsLayoutItemMap *> mapItems;
@@ -328,54 +356,50 @@ void QgsLayoutGuiUtils::registerGuiForKnownItemTypes( QgsMapCanvas *mapCanvas )
   // shape items
 
   auto createShapeWidget =
-    []( QgsLayoutItem * item )->QgsLayoutItemBaseWidget *
-  {
-    return new QgsLayoutShapeWidget( qobject_cast< QgsLayoutItemShape * >( item ) );
+    []( QgsLayoutItem *item ) -> QgsLayoutItemBaseWidget * {
+    return new QgsLayoutShapeWidget( qobject_cast<QgsLayoutItemShape *>( item ) );
   };
 
-  registry->addLayoutItemGuiMetadata( new QgsLayoutItemGuiMetadata( QgsLayoutItemRegistry::LayoutShape, QObject::tr( "Rectangle" ), QgsApplication::getThemeIcon( QStringLiteral( "/mActionAddBasicRectangle.svg" ) ), createShapeWidget, createRubberBand, QStringLiteral( "shapes" ), false, QgsLayoutItemAbstractGuiMetadata::Flags(), []( QgsLayout * layout )->QgsLayoutItem*
-  {
-    std::unique_ptr< QgsLayoutItemShape > shape = std::make_unique< QgsLayoutItemShape >( layout );
+  registry->addLayoutItemGuiMetadata( new QgsLayoutItemGuiMetadata( QgsLayoutItemRegistry::LayoutShape, QObject::tr( "Rectangle" ), QgsApplication::getThemeIcon( QStringLiteral( "/mActionAddBasicRectangle.svg" ) ), createShapeWidget, createRubberBand, QStringLiteral( "shapes" ), false, QgsLayoutItemAbstractGuiMetadata::Flags(), []( QgsLayout *layout ) -> QgsLayoutItem * {
+    auto shape = std::make_unique<QgsLayoutItemShape>( layout );
     shape->setShapeType( QgsLayoutItemShape::Rectangle );
     return shape.release();
   } ) );
-  registry->addLayoutItemGuiMetadata( new QgsLayoutItemGuiMetadata( QgsLayoutItemRegistry::LayoutShape, QObject::tr( "Ellipse" ), QgsApplication::getThemeIcon( QStringLiteral( "/mActionAddBasicCircle.svg" ) ), createShapeWidget, createEllipseBand, QStringLiteral( "shapes" ), false, QgsLayoutItemAbstractGuiMetadata::Flags(), []( QgsLayout * layout )->QgsLayoutItem*
-  {
-    std::unique_ptr< QgsLayoutItemShape > shape = std::make_unique< QgsLayoutItemShape >( layout );
+  registry->addLayoutItemGuiMetadata( new QgsLayoutItemGuiMetadata( QgsLayoutItemRegistry::LayoutShape, QObject::tr( "Ellipse" ), QgsApplication::getThemeIcon( QStringLiteral( "/mActionAddBasicCircle.svg" ) ), createShapeWidget, createEllipseBand, QStringLiteral( "shapes" ), false, QgsLayoutItemAbstractGuiMetadata::Flags(), []( QgsLayout *layout ) -> QgsLayoutItem * {
+    auto shape = std::make_unique<QgsLayoutItemShape>( layout );
     shape->setShapeType( QgsLayoutItemShape::Ellipse );
     return shape.release();
   } ) );
-  registry->addLayoutItemGuiMetadata( new QgsLayoutItemGuiMetadata( QgsLayoutItemRegistry::LayoutShape, QObject::tr( "Triangle" ), QgsApplication::getThemeIcon( QStringLiteral( "/mActionAddBasicTriangle.svg" ) ), createShapeWidget, createTriangleBand, QStringLiteral( "shapes" ), false, QgsLayoutItemAbstractGuiMetadata::Flags(), []( QgsLayout * layout )->QgsLayoutItem*
-  {
-    std::unique_ptr< QgsLayoutItemShape > shape = std::make_unique< QgsLayoutItemShape >( layout );
+  registry->addLayoutItemGuiMetadata( new QgsLayoutItemGuiMetadata( QgsLayoutItemRegistry::LayoutShape, QObject::tr( "Triangle" ), QgsApplication::getThemeIcon( QStringLiteral( "/mActionAddBasicTriangle.svg" ) ), createShapeWidget, createTriangleBand, QStringLiteral( "shapes" ), false, QgsLayoutItemAbstractGuiMetadata::Flags(), []( QgsLayout *layout ) -> QgsLayoutItem * {
+    auto shape = std::make_unique<QgsLayoutItemShape>( layout );
     shape->setShapeType( QgsLayoutItemShape::Triangle );
     return shape.release();
   } ) );
 
   // marker
-  registry->addLayoutItemGuiMetadata( new QgsLayoutItemGuiMetadata( QgsLayoutItemRegistry::LayoutMarker, QObject::tr( "Marker" ), QgsApplication::getThemeIcon( QStringLiteral( "/mActionAddMarker.svg" ) ),
-                                      [ = ]( QgsLayoutItem * item )->QgsLayoutItemBaseWidget *
-  {
-    return new QgsLayoutMarkerWidget( qobject_cast< QgsLayoutItemMarker * >( item ) );
-  }, nullptr ) );
+  registry->addLayoutItemGuiMetadata( new QgsLayoutItemGuiMetadata( QgsLayoutItemRegistry::LayoutMarker, QObject::tr( "Marker" ), QgsApplication::getThemeIcon( QStringLiteral( "/mActionAddMarker.svg" ) ), [=]( QgsLayoutItem *item ) -> QgsLayoutItemBaseWidget * { return new QgsLayoutMarkerWidget( qobject_cast<QgsLayoutItemMarker *>( item ) ); }, nullptr ) );
 
   // arrow
-  std::unique_ptr< QgsLayoutItemGuiMetadata > arrowMetadata = std::make_unique< QgsLayoutItemGuiMetadata>(
-        QgsLayoutItemRegistry::LayoutPolyline, QObject::tr( "Arrow" ), QgsApplication::getThemeIcon( QStringLiteral( "/mActionAddArrow.svg" ) ),
-        [ = ]( QgsLayoutItem * item )->QgsLayoutItemBaseWidget *
-  {
-    return new QgsLayoutPolylineWidget( qobject_cast< QgsLayoutItemPolyline * >( item ) );
-  }, createRubberBand, QString(), true );
-  arrowMetadata->setItemCreationFunction( []( QgsLayout * layout )->QgsLayoutItem *
-  {
-    std::unique_ptr< QgsLayoutItemPolyline > arrow = std::make_unique< QgsLayoutItemPolyline >( layout );
+  auto arrowMetadata = std::make_unique<QgsLayoutItemGuiMetadata>(
+    QgsLayoutItemRegistry::LayoutPolyline, QObject::tr( "Arrow" ), QgsApplication::getThemeIcon( QStringLiteral( "/mActionAddArrow.svg" ) ),
+    [=]( QgsLayoutItem *item ) -> QgsLayoutItemBaseWidget * {
+      return new QgsLayoutPolylineWidget( qobject_cast<QgsLayoutItemPolyline *>( item ) );
+    },
+    createRubberBand, QString(), true
+  );
+  arrowMetadata->setItemCreationFunction( []( QgsLayout *layout ) -> QgsLayoutItem * {
+    auto arrow = std::make_unique<QgsLayoutItemPolyline>( layout );
     arrow->setEndMarker( QgsLayoutItemPolyline::ArrowHead );
     return arrow.release();
   } );
-  arrowMetadata->setNodeRubberBandCreationFunction( []( QgsLayoutView * )->QGraphicsPathItem*
-  {
-    std::unique_ptr< QGraphicsPathItem > band = std::make_unique< QGraphicsPathItem >();
-    band->setPen( QPen( QBrush( QColor( 227, 22, 22, 200 ) ), 0 ) );
+  arrowMetadata->setNodeRubberBandCreationFunction( []( QgsLayoutView * ) -> QGraphicsItemGroup * {
+    auto band = std::make_unique<QGraphicsItemGroup>();
+    QGraphicsPathItem *poly = new QGraphicsPathItem( band.get() );
+    poly->setPen( QPen( QBrush( QColor( 227, 22, 22, 200 ) ), 0 ) );
+
+    QGraphicsPathItem *tempPoly = new QGraphicsPathItem( band.get() );
+    tempPoly->setPen( QPen( QBrush( QColor( 227, 22, 22, 200 ) ), 0, Qt::DotLine ) );
+
     band->setZValue( QgsLayout::ZViewTool );
     return band.release();
   } );
@@ -383,32 +407,43 @@ void QgsLayoutGuiUtils::registerGuiForKnownItemTypes( QgsMapCanvas *mapCanvas )
 
   // node items
 
-  std::unique_ptr< QgsLayoutItemGuiMetadata > polygonMetadata = std::make_unique< QgsLayoutItemGuiMetadata >(
-        QgsLayoutItemRegistry::LayoutPolygon, QObject::tr( "Polygon" ), QgsApplication::getThemeIcon( QStringLiteral( "/mActionAddPolygon.svg" ) ),
-        [ = ]( QgsLayoutItem * item )->QgsLayoutItemBaseWidget *
-  {
-    return new QgsLayoutPolygonWidget( qobject_cast< QgsLayoutItemPolygon * >( item ) );
-  }, createRubberBand, QStringLiteral( "nodes" ), true );
-  polygonMetadata->setNodeRubberBandCreationFunction( []( QgsLayoutView * )->QGraphicsPolygonItem*
-  {
-    std::unique_ptr< QGraphicsPolygonItem > band = std::make_unique< QGraphicsPolygonItem >();
-    band->setBrush( Qt::NoBrush );
-    band->setPen( QPen( QBrush( QColor( 227, 22, 22, 200 ) ), 0 ) );
+  auto polygonMetadata = std::make_unique<QgsLayoutItemGuiMetadata>(
+    QgsLayoutItemRegistry::LayoutPolygon, QObject::tr( "Polygon" ), QgsApplication::getThemeIcon( QStringLiteral( "/mActionAddPolygon.svg" ) ),
+    [=]( QgsLayoutItem *item ) -> QgsLayoutItemBaseWidget * {
+      return new QgsLayoutPolygonWidget( qobject_cast<QgsLayoutItemPolygon *>( item ) );
+    },
+    createRubberBand, QStringLiteral( "nodes" ), true
+  );
+  polygonMetadata->setNodeRubberBandCreationFunction( []( QgsLayoutView * ) -> QGraphicsItemGroup * {
+    auto band = std::make_unique<QGraphicsItemGroup>();
+    QGraphicsPolygonItem *poly = new QGraphicsPolygonItem( band.get() );
+    poly->setBrush( QBrush( QColor( 227, 22, 22, 20 ) ) );
+    poly->setPen( QPen( QBrush( QColor( 227, 22, 22, 200 ) ), 0 ) );
+
+    QGraphicsPolygonItem *tempPoly = new QGraphicsPolygonItem( band.get() );
+    tempPoly->setBrush( Qt::NoBrush );
+    tempPoly->setPen( QPen( QBrush( QColor( 227, 22, 22, 200 ) ), 0, Qt::DotLine ) );
+
     band->setZValue( QgsLayout::ZViewTool );
     return band.release();
   } );
   registry->addLayoutItemGuiMetadata( polygonMetadata.release() );
 
-  std::unique_ptr< QgsLayoutItemGuiMetadata > polylineMetadata = std::make_unique< QgsLayoutItemGuiMetadata>(
-        QgsLayoutItemRegistry::LayoutPolyline, QObject::tr( "Polyline" ), QgsApplication::getThemeIcon( QStringLiteral( "/mActionAddPolyline.svg" ) ),
-        [ = ]( QgsLayoutItem * item )->QgsLayoutItemBaseWidget *
-  {
-    return new QgsLayoutPolylineWidget( qobject_cast< QgsLayoutItemPolyline * >( item ) );
-  }, createRubberBand, QStringLiteral( "nodes" ), true );
-  polylineMetadata->setNodeRubberBandCreationFunction( []( QgsLayoutView * )->QGraphicsPathItem*
-  {
-    std::unique_ptr< QGraphicsPathItem > band = std::make_unique< QGraphicsPathItem >();
-    band->setPen( QPen( QBrush( QColor( 227, 22, 22, 200 ) ), 0 ) );
+  auto polylineMetadata = std::make_unique<QgsLayoutItemGuiMetadata>(
+    QgsLayoutItemRegistry::LayoutPolyline, QObject::tr( "Polyline" ), QgsApplication::getThemeIcon( QStringLiteral( "/mActionAddPolyline.svg" ) ),
+    [=]( QgsLayoutItem *item ) -> QgsLayoutItemBaseWidget * {
+      return new QgsLayoutPolylineWidget( qobject_cast<QgsLayoutItemPolyline *>( item ) );
+    },
+    createRubberBand, QStringLiteral( "nodes" ), true
+  );
+  polylineMetadata->setNodeRubberBandCreationFunction( []( QgsLayoutView * ) -> QGraphicsItemGroup * {
+    auto band = std::make_unique<QGraphicsItemGroup>();
+    QGraphicsPathItem *poly = new QGraphicsPathItem( band.get() );
+    poly->setPen( QPen( QBrush( QColor( 227, 22, 22, 200 ) ), 0 ) );
+
+    QGraphicsPathItem *tempPoly = new QGraphicsPathItem( band.get() );
+    tempPoly->setPen( QPen( QBrush( QColor( 227, 22, 22, 200 ) ), 0, Qt::DotLine ) );
+
     band->setZValue( QgsLayout::ZViewTool );
     return band.release();
   } );
@@ -417,17 +452,12 @@ void QgsLayoutGuiUtils::registerGuiForKnownItemTypes( QgsMapCanvas *mapCanvas )
 
   // html item
 
-  auto htmlItemMetadata = std::make_unique< QgsLayoutItemGuiMetadata >( QgsLayoutItemRegistry::LayoutHtml, QObject::tr( "HTML" ), QgsApplication::getThemeIcon( QStringLiteral( "/mActionAddHtml.svg" ) ),
-                          [ = ]( QgsLayoutItem * item )->QgsLayoutItemBaseWidget *
-  {
-    return new QgsLayoutHtmlWidget( qobject_cast< QgsLayoutFrame * >( item ) );
-  }, createRubberBand );
-  htmlItemMetadata->setItemCreationFunction( [ = ]( QgsLayout * layout )->QgsLayoutItem *
-  {
-    std::unique_ptr< QgsLayoutItemHtml > htmlMultiFrame = std::make_unique< QgsLayoutItemHtml >( layout );
+  auto htmlItemMetadata = std::make_unique<QgsLayoutItemGuiMetadata>( QgsLayoutItemRegistry::LayoutHtml, QObject::tr( "HTML" ), QgsApplication::getThemeIcon( QStringLiteral( "/mActionAddHtml.svg" ) ), [=]( QgsLayoutItem *item ) -> QgsLayoutItemBaseWidget * { return new QgsLayoutHtmlWidget( qobject_cast<QgsLayoutFrame *>( item ) ); }, createRubberBand );
+  htmlItemMetadata->setItemCreationFunction( [=]( QgsLayout *layout ) -> QgsLayoutItem * {
+    auto htmlMultiFrame = std::make_unique<QgsLayoutItemHtml>( layout );
     QgsLayoutItemHtml *html = htmlMultiFrame.get();
     layout->addMultiFrame( htmlMultiFrame.release() );
-    std::unique_ptr< QgsLayoutFrame > frame = std::make_unique< QgsLayoutFrame >( layout, html );
+    auto frame = std::make_unique<QgsLayoutFrame>( layout, html );
     QgsLayoutFrame *f = frame.get();
     html->addFrame( frame.release() );
     // cppcheck-suppress returnDanglingLifetime
@@ -437,19 +467,14 @@ void QgsLayoutGuiUtils::registerGuiForKnownItemTypes( QgsMapCanvas *mapCanvas )
 
   // attribute table item
 
-  auto attributeTableItemMetadata = std::make_unique< QgsLayoutItemGuiMetadata >( QgsLayoutItemRegistry::LayoutAttributeTable, QObject::tr( "Attribute Table" ), QgsApplication::getThemeIcon( QStringLiteral( "/mActionAddTable.svg" ) ),
-                                    [ = ]( QgsLayoutItem * item )->QgsLayoutItemBaseWidget *
-  {
-    return new QgsLayoutAttributeTableWidget( qobject_cast< QgsLayoutFrame * >( item ) );
-  }, createRubberBand );
-  attributeTableItemMetadata->setItemCreationFunction( [ = ]( QgsLayout * layout )->QgsLayoutItem *
-  {
-    std::unique_ptr< QgsLayoutItemAttributeTable > tableMultiFrame = std::make_unique< QgsLayoutItemAttributeTable >( layout );
+  auto attributeTableItemMetadata = std::make_unique<QgsLayoutItemGuiMetadata>( QgsLayoutItemRegistry::LayoutAttributeTable, QObject::tr( "Attribute Table" ), QgsApplication::getThemeIcon( QStringLiteral( "/mActionAddTable.svg" ) ), [=]( QgsLayoutItem *item ) -> QgsLayoutItemBaseWidget * { return new QgsLayoutAttributeTableWidget( qobject_cast<QgsLayoutFrame *>( item ) ); }, createRubberBand );
+  attributeTableItemMetadata->setItemCreationFunction( [=]( QgsLayout *layout ) -> QgsLayoutItem * {
+    auto tableMultiFrame = std::make_unique<QgsLayoutItemAttributeTable>( layout );
     QgsLayoutItemAttributeTable *table = tableMultiFrame.get();
 
     //set first vector layer from layer registry as table source
     QMap<QString, QgsMapLayer *> layerMap = layout->project()->mapLayers();
-    for ( auto it = layerMap.constBegin() ; it != layerMap.constEnd(); ++it )
+    for ( auto it = layerMap.constBegin(); it != layerMap.constEnd(); ++it )
     {
       if ( QgsVectorLayer *vl = qobject_cast<QgsVectorLayer *>( it.value() ) )
       {
@@ -474,7 +499,7 @@ void QgsLayoutGuiUtils::registerGuiForKnownItemTypes( QgsMapCanvas *mapCanvas )
     }
 
     layout->addMultiFrame( tableMultiFrame.release() );
-    std::unique_ptr< QgsLayoutFrame > frame = std::make_unique< QgsLayoutFrame >( layout, table );
+    auto frame = std::make_unique<QgsLayoutFrame>( layout, table );
     QgsLayoutFrame *f = frame.get();
     table->addFrame( frame.release() );
     // cppcheck-suppress returnDanglingLifetime
@@ -484,14 +509,9 @@ void QgsLayoutGuiUtils::registerGuiForKnownItemTypes( QgsMapCanvas *mapCanvas )
 
   // manual table item
 
-  auto manualTableItemMetadata = std::make_unique< QgsLayoutItemGuiMetadata >( QgsLayoutItemRegistry::LayoutManualTable, QObject::tr( "Fixed Table" ), QgsApplication::getThemeIcon( QStringLiteral( "/mActionAddManualTable.svg" ) ),
-                                 [ = ]( QgsLayoutItem * item )->QgsLayoutItemBaseWidget *
-  {
-    return new QgsLayoutManualTableWidget( qobject_cast< QgsLayoutFrame * >( item ) );
-  }, createRubberBand );
-  manualTableItemMetadata->setItemCreationFunction( [ = ]( QgsLayout * layout )->QgsLayoutItem *
-  {
-    std::unique_ptr< QgsLayoutItemManualTable > tableMultiFrame = std::make_unique< QgsLayoutItemManualTable >( layout );
+  auto manualTableItemMetadata = std::make_unique<QgsLayoutItemGuiMetadata>( QgsLayoutItemRegistry::LayoutManualTable, QObject::tr( "Fixed Table" ), QgsApplication::getThemeIcon( QStringLiteral( "/mActionAddManualTable.svg" ) ), [=]( QgsLayoutItem *item ) -> QgsLayoutItemBaseWidget * { return new QgsLayoutManualTableWidget( qobject_cast<QgsLayoutFrame *>( item ) ); }, createRubberBand );
+  manualTableItemMetadata->setItemCreationFunction( [=]( QgsLayout *layout ) -> QgsLayoutItem * {
+    auto tableMultiFrame = std::make_unique<QgsLayoutItemManualTable>( layout );
     QgsLayoutItemManualTable *table = tableMultiFrame.get();
 
     // initially start with a 2x2 empty table
@@ -517,25 +537,23 @@ void QgsLayoutGuiUtils::registerGuiForKnownItemTypes( QgsMapCanvas *mapCanvas )
 
     layout->addMultiFrame( tableMultiFrame.release() );
 
-    std::unique_ptr< QgsLayoutFrame > frame = std::make_unique< QgsLayoutFrame >( layout, table );
+    auto frame = std::make_unique<QgsLayoutFrame>( layout, table );
     QgsLayoutFrame *f = frame.get();
     table->addFrame( frame.release() );
     // cppcheck-suppress returnDanglingLifetime
     return f;
   } );
+  manualTableItemMetadata->setItemDoubleClickedFunction( [=]( QgsLayoutItem *item, Qgis::MouseHandlesAction ) {
+    QgsLayoutManualTableWidget::openTableDesigner( qobject_cast<QgsLayoutFrame *>( item ) );
+  } );
   registry->addLayoutItemGuiMetadata( manualTableItemMetadata.release() );
 
 
   // elevation profile item
 
-  auto elevationProfileItemMetadata = std::make_unique< QgsLayoutItemGuiMetadata >( QgsLayoutItemRegistry::LayoutElevationProfile, QObject::tr( "Elevation Profile" ), QgsApplication::getThemeIcon( QStringLiteral( "/mActionElevationProfile.svg" ) ),
-                                      [ = ]( QgsLayoutItem * item )->QgsLayoutItemBaseWidget *
-  {
-    return new QgsLayoutElevationProfileWidget( qobject_cast< QgsLayoutItemElevationProfile * >( item ) );
-  }, createRubberBand );
-  elevationProfileItemMetadata->setItemCreationFunction( [ = ]( QgsLayout * layout )->QgsLayoutItem *
-  {
-    std::unique_ptr< QgsLayoutItemElevationProfile > profileItem = std::make_unique< QgsLayoutItemElevationProfile >( layout );
+  auto elevationProfileItemMetadata = std::make_unique<QgsLayoutItemGuiMetadata>( QgsLayoutItemRegistry::LayoutElevationProfile, QObject::tr( "Elevation Profile" ), QgsApplication::getThemeIcon( QStringLiteral( "/mActionElevationProfile.svg" ) ), [=]( QgsLayoutItem *item ) -> QgsLayoutItemBaseWidget * { return new QgsLayoutElevationProfileWidget( qobject_cast<QgsLayoutItemElevationProfile *>( item ) ); }, createRubberBand );
+  elevationProfileItemMetadata->setItemCreationFunction( [=]( QgsLayout *layout ) -> QgsLayoutItem * {
+    auto profileItem = std::make_unique<QgsLayoutItemElevationProfile>( layout );
 
     //set default fonts from settings
     QgsSettings settings;
@@ -557,5 +575,4 @@ void QgsLayoutGuiUtils::registerGuiForKnownItemTypes( QgsMapCanvas *mapCanvas )
     return profileItem.release();
   } );
   registry->addLayoutItemGuiMetadata( elevationProfileItemMetadata.release() );
-
 }
