@@ -14,18 +14,24 @@
  * (at your option) any later version.
  *
  ***************************************************************************/
-#include "qgshanaconnection.h"
+
 #include "qgshanadataitems.h"
 #include "qgshanadataitemguiprovider.h"
 #include "moc_qgshanadataitemguiprovider.cpp"
+#include "qgsapplication.h"
 #include "qgshananewconnection.h"
-#include "qgshanaprovider.h"
 #include "qgshanaproviderconnection.h"
 #include "qgshanasourceselect.h"
-#include "qgshanautils.h"
 #include "qgsnewnamedialog.h"
 #include "qgsdataitemguiproviderutils.h"
 #include "qgssettings.h"
+#include "qgshanautils.h"
+#include "qgsapplication.h"
+#include "qgstaskmanager.h"
+#include "qgsvectorlayer.h"
+#include "qgsvectorlayerexporter.h"
+#include "qgsmessageoutput.h"
+#include "qgsdbimportvectorlayerdialog.h"
 
 #include <QInputDialog>
 #include <QMessageBox>
@@ -43,36 +49,48 @@ void QgsHanaDataItemGuiProvider::populateContextMenu(
 
   if ( QgsHanaConnectionItem *connItem = qobject_cast<QgsHanaConnectionItem *>( item ) )
   {
-    QAction *actionRefresh = new QAction( tr( "Refresh" ), this );
-    connect( actionRefresh, &QAction::triggered, this, [connItem] { refreshConnection( connItem ); } );
-    menu->addAction( actionRefresh );
-
-    menu->addSeparator();
-
-    QAction *actionEdit = new QAction( tr( "Edit Connection…" ), this );
-    connect( actionEdit, &QAction::triggered, this, [connItem] { editConnection( connItem ); } );
-    menu->addAction( actionEdit );
-
-    QAction *actionDuplicate = new QAction( tr( "Duplicate Connection" ), this );
-    connect( actionDuplicate, &QAction::triggered, this, [connItem] { duplicateConnection( connItem ); } );
-    menu->addAction( actionDuplicate );
-
     const QList<QgsHanaConnectionItem *> hanaConnectionItems = QgsDataItem::filteredItems<QgsHanaConnectionItem>( selection );
+
+    if ( hanaConnectionItems.size() == 1 )
+    {
+      QAction *actionRefresh = new QAction( tr( "Refresh" ), this );
+      connect( actionRefresh, &QAction::triggered, this, [connItem] { refreshConnection( connItem ); } );
+      menu->addAction( actionRefresh );
+
+      menu->addSeparator();
+
+      QAction *actionEdit = new QAction( tr( "Edit Connection…" ), this );
+      connect( actionEdit, &QAction::triggered, this, [connItem] { editConnection( connItem ); } );
+      menu->addAction( actionEdit );
+
+      QAction *actionDuplicate = new QAction( tr( "Duplicate Connection" ), this );
+      connect( actionDuplicate, &QAction::triggered, this, [connItem] { duplicateConnection( connItem ); } );
+      menu->addAction( actionDuplicate );
+    }
+
     QAction *actionDelete = new QAction( hanaConnectionItems.size() > 1 ? tr( "Remove Connections…" ) : tr( "Remove Connection…" ), menu );
     connect( actionDelete, &QAction::triggered, this, [hanaConnectionItems, context] {
       QgsDataItemGuiProviderUtils::deleteConnections( hanaConnectionItems, []( const QString &connectionName ) { QgsHanaSettings::removeConnection( connectionName ); }, context );
     } );
     menu->addAction( actionDelete );
 
-    menu->addSeparator();
-
-    QAction *actionCreateSchema = new QAction( tr( "New Schema…" ), this );
-    connect( actionCreateSchema, &QAction::triggered, this, [connItem, context] { createSchema( connItem, context ); } );
-    menu->addAction( actionCreateSchema );
+    if ( hanaConnectionItems.size() == 1 )
+    {
+      menu->addSeparator();
+      QAction *actionCreateSchema = new QAction( tr( "New Schema…" ), this );
+      connect( actionCreateSchema, &QAction::triggered, this, [connItem, context] { createSchema( connItem, context ); } );
+      menu->addAction( actionCreateSchema );
+    }
   }
 
   if ( QgsHanaSchemaItem *schemaItem = qobject_cast<QgsHanaSchemaItem *>( item ) )
   {
+    QAction *importVectorAction = new QAction( QObject::tr( "Import Vector Layer…" ), menu );
+    menu->addAction( importVectorAction );
+    const QString destinationSchema = schemaItem->name();
+    QgsHanaConnectionItem *connItem = qobject_cast<QgsHanaConnectionItem *>( schemaItem->parent() );
+    QObject::connect( importVectorAction, &QAction::triggered, item, [connItem, context, destinationSchema, this] { handleImportVector( connItem, destinationSchema, context ); } );
+
     QAction *actionRefresh = new QAction( tr( "Refresh" ), this );
     connect( actionRefresh, &QAction::triggered, this, [schemaItem] { schemaItem->refresh(); } );
     menu->addAction( actionRefresh );
@@ -157,12 +175,12 @@ bool QgsHanaDataItemGuiProvider::acceptDrop( QgsDataItem *item, QgsDataItemGuiCo
 }
 
 bool QgsHanaDataItemGuiProvider::handleDrop(
-  QgsDataItem *item, QgsDataItemGuiContext, const QMimeData *data, Qt::DropAction
+  QgsDataItem *item, QgsDataItemGuiContext context, const QMimeData *data, Qt::DropAction
 )
 {
   if ( QgsHanaConnectionItem *connItem = qobject_cast<QgsHanaConnectionItem *>( item ) )
   {
-    return connItem->handleDrop( data, QString() );
+    return handleDrop( connItem, data, QString(), context );
   }
   else if ( QgsHanaSchemaItem *schemaItem = qobject_cast<QgsHanaSchemaItem *>( item ) )
   {
@@ -170,7 +188,7 @@ bool QgsHanaDataItemGuiProvider::handleDrop(
     if ( !connItem )
       return false;
 
-    return connItem->handleDrop( data, schemaItem->name() );
+    return handleDrop( connItem, data, schemaItem->name(), context );
   }
   return false;
 }
@@ -187,7 +205,7 @@ QWidget *QgsHanaDataItemGuiProvider::createParamWidget( QgsDataItem *root, QgsDa
 
 void QgsHanaDataItemGuiProvider::newConnection( QgsDataItem *item )
 {
-  QgsHanaNewConnection nc( nullptr );
+  QgsHanaNewConnection nc( QgsApplication::instance()->activeWindow() );
   if ( nc.exec() )
   {
     item->refresh();
@@ -381,3 +399,171 @@ void QgsHanaDataItemGuiProvider::renameLayer( QgsHanaLayerItem *layerItem, QgsDa
     notify( caption, tr( "Unable to rename '%1'\n%2" ).arg( layerInfo.tableName, errorMsg ), context, Qgis::MessageLevel::Warning );
   }
 }
+
+bool QgsHanaDataItemGuiProvider::handleDrop( QgsHanaConnectionItem *connectionItem, const QMimeData *data, const QString &toSchema, QgsDataItemGuiContext context )
+{
+  if ( !QgsMimeDataUtils::isUriList( data ) || !connectionItem )
+    return false;
+
+  const QgsMimeDataUtils::UriList sourceUris = QgsMimeDataUtils::decodeUriList( data );
+  if ( sourceUris.size() == 1 && sourceUris.at( 0 ).layerType == QLatin1String( "vector" ) )
+  {
+    return handleDropUri( connectionItem, sourceUris.at( 0 ), toSchema, context );
+  }
+
+  QStringList importResults;
+  bool hasError = false;
+
+  QPointer< QgsHanaConnectionItem > connectionItemPointer( connectionItem );
+
+  QgsDataSourceUri uri = connectionItem->connectionUri();
+  QgsHanaConnectionRef conn( uri );
+
+  // TODO: when dropping multiple layers, we need a dedicated "bulk import" dialog for settings which apply to ALL layers
+  std::unique_ptr<QgsAbstractDatabaseProviderConnection> databaseConnection( connectionItem->databaseConnection() );
+  if ( !databaseConnection )
+    return false;
+
+  if ( !conn.isNull() )
+  {
+    const QgsMimeDataUtils::UriList lst = QgsMimeDataUtils::decodeUriList( data );
+    for ( const QgsMimeDataUtils::Uri &u : lst )
+    {
+      if ( u.layerType != QLatin1String( "vector" ) )
+      {
+        importResults.append( tr( "%1: Not a vector layer!" ).arg( u.name ) );
+        hasError = true; // only vectors can be imported
+        continue;
+      }
+
+      // open the source layer
+      bool owner;
+      QString error;
+      QgsVectorLayer *srcLayer = u.vectorLayer( owner, error );
+      if ( !srcLayer )
+      {
+        importResults.append( QStringLiteral( "%1: %2" ).arg( u.name, error ) );
+        hasError = true;
+        continue;
+      }
+
+      if ( srcLayer->isValid() )
+      {
+        QgsDataSourceUri dsUri( u.uri );
+        QString geomColumn = dsUri.geometryColumn();
+        if ( geomColumn.isEmpty() )
+        {
+          bool fieldsInUpperCase = QgsHanaUtils::countFieldsWithFirstLetterInUppercase( srcLayer->fields() ) > srcLayer->fields().size() / 2;
+          geomColumn = ( srcLayer->geometryType() != Qgis::GeometryType::Null ) ? ( fieldsInUpperCase ? QStringLiteral( "GEOM" ) : QStringLiteral( "geom" ) ) : nullptr;
+        }
+
+        QgsAbstractDatabaseProviderConnection::VectorLayerExporterOptions exporterOptions;
+        exporterOptions.layerName = u.name;
+        exporterOptions.schema = toSchema;
+        exporterOptions.wkbType = srcLayer->wkbType();
+        exporterOptions.geometryColumn = geomColumn;
+        exporterOptions.primaryKeyColumns << dsUri.keyColumn();
+
+        QVariantMap providerOptions;
+        const QString destUri = databaseConnection->createVectorLayerExporterDestinationUri( exporterOptions, providerOptions );
+
+        std::unique_ptr<QgsVectorLayerExporterTask> exportTask(
+          new QgsVectorLayerExporterTask( srcLayer, destUri, QStringLiteral( "hana" ), srcLayer->crs(), providerOptions, owner )
+        );
+
+        // when export is successful:
+        connect( exportTask.get(), &QgsVectorLayerExporterTask::exportComplete, this, [=]() {
+          QMessageBox::information( nullptr, tr( "Import to SAP HANA database" ), tr( "Import was successful." ) );
+          if ( connectionItemPointer )
+            connectionItemPointer->refreshSchema( toSchema );
+        } );
+
+        // when an error occurs:
+        connect( exportTask.get(), &QgsVectorLayerExporterTask::errorOccurred, this, [=]( Qgis::VectorExportResult error, const QString &errorMessage ) {
+          if ( error != Qgis::VectorExportResult::UserCanceled )
+          {
+            QgsMessageOutput *output = QgsMessageOutput::createMessageOutput();
+            output->setTitle( tr( "Import to SAP HANA database" ) );
+            output->setMessage( tr( "Failed to import some layers!\n\n" ) + errorMessage, QgsMessageOutput::MessageText );
+            output->showMessage();
+          }
+          if ( connectionItemPointer )
+            connectionItemPointer->refreshSchema( toSchema );
+        } );
+
+        QgsApplication::taskManager()->addTask( exportTask.release() );
+      }
+      else
+      {
+        importResults.append( tr( "%1: Not a valid layer!" ).arg( u.name ) );
+        hasError = true;
+      }
+    }
+  }
+  else
+  {
+    importResults.append( tr( "Connection failed" ) );
+    hasError = true;
+  }
+
+  if ( hasError )
+  {
+    QgsMessageOutput *output = QgsMessageOutput::createMessageOutput();
+    output->setTitle( tr( "Import to SAP HANA database" ) );
+    output->setMessage( tr( "Failed to import some layers!\n\n" ) + importResults.join( QLatin1Char( '\n' ) ), QgsMessageOutput::MessageText );
+    output->showMessage();
+  }
+
+  return true;
+}
+
+bool QgsHanaDataItemGuiProvider::handleDropUri( QgsHanaConnectionItem *connectionItem, const QgsMimeDataUtils::Uri &sourceUri, const QString &toSchema, QgsDataItemGuiContext context )
+{
+  QPointer< QgsHanaConnectionItem > connectionItemPointer( connectionItem );
+  std::unique_ptr<QgsAbstractDatabaseProviderConnection> databaseConnection( connectionItem->databaseConnection() );
+  if ( !databaseConnection )
+    return false;
+
+  auto onSuccess = [connectionItemPointer, toSchema]() {
+    if ( connectionItemPointer )
+    {
+      connectionItemPointer->refreshSchema( toSchema );
+    }
+  };
+
+  auto onFailure = [connectionItemPointer = std::move( connectionItemPointer ), toSchema]( Qgis::VectorExportResult, const QString & ) {
+    if ( connectionItemPointer )
+    {
+      connectionItemPointer->refreshSchema( toSchema );
+    }
+  };
+
+  return QgsDataItemGuiProviderUtils::handleDropUriForConnection( std::move( databaseConnection ), sourceUri, toSchema, context, tr( "SAP HANA Import" ), tr( "Import to SAP HANA database" ), QVariantMap(), onSuccess, onFailure, this );
+}
+
+void QgsHanaDataItemGuiProvider::handleImportVector( QgsHanaConnectionItem *connectionItem, const QString &toSchema, QgsDataItemGuiContext context )
+{
+  if ( !connectionItem )
+    return;
+
+  QPointer< QgsHanaConnectionItem > connectionItemPointer( connectionItem );
+  std::unique_ptr<QgsAbstractDatabaseProviderConnection> databaseConnection( connectionItem->databaseConnection() );
+  if ( !databaseConnection )
+    return;
+
+  auto onSuccess = [connectionItemPointer, toSchema]() {
+    if ( connectionItemPointer )
+    {
+      connectionItemPointer->refreshSchema( toSchema );
+    }
+  };
+
+  auto onFailure = [connectionItemPointer = std::move( connectionItemPointer ), toSchema]( Qgis::VectorExportResult, const QString & ) {
+    if ( connectionItemPointer )
+    {
+      connectionItemPointer->refreshSchema( toSchema );
+    }
+  };
+
+  QgsDataItemGuiProviderUtils::handleImportVectorLayerForConnection( std::move( databaseConnection ), toSchema, context, tr( "SAP HANA Import" ), tr( "Import to SAP HANA database" ), QVariantMap(), onSuccess, onFailure, this );
+}
