@@ -42,11 +42,14 @@ class TestQgsMapToolReshape : public QObject
     void initTestCase();    // will be called before the first testfunction is executed.
     void cleanupTestCase(); // will be called after the last testfunction was executed.
 
+    void testReshapeNoChange();
     void testReshapeZ();
     void testTopologicalEditing();
+    void testTopologicalEditingNoSnap();
     void testAvoidIntersectionAndTopoEdit();
     void reshapeWithBindingLine();
     void testWithTracing();
+    void testKeepDirection();
 
   private:
     QgisApp *mQgisApp = nullptr;
@@ -57,6 +60,7 @@ class TestQgsMapToolReshape : public QObject
     QgsVectorLayer *mLayerPolygonZ = nullptr;
     QgsVectorLayer *mLayerTopo = nullptr;
     QgsVectorLayer *mLayerTopo2 = nullptr;
+    QgsVectorLayer *mLayerLine = nullptr;
 };
 
 TestQgsMapToolReshape::TestQgsMapToolReshape() = default;
@@ -106,6 +110,10 @@ void TestQgsMapToolReshape::initTestCase()
   mLayerTopo2 = new QgsVectorLayer( QStringLiteral( "Polygon?crs=EPSG:3946" ), QStringLiteral( "topo2" ), QStringLiteral( "memory" ) );
   QVERIFY( mLayerTopo2->isValid() );
 
+  mLayerLine = new QgsVectorLayer( QStringLiteral( "LineString?crs=EPSG:3946" ), QStringLiteral( "layer line" ), QStringLiteral( "memory" ) );
+  QVERIFY( mLayerLine->isValid() );
+  QgsProject::instance()->addMapLayers( QList<QgsMapLayer *>() << mLayerLine );
+
   mLayerLineZ->startEditing();
   const QString wkt1 = "LineString Z (0 0 0, 1 1 0, 1 2 0)";
   const QString wkt2 = "LineString Z (2 1 5, 3 3 5)";
@@ -147,10 +155,10 @@ void TestQgsMapToolReshape::initTestCase()
   QCOMPARE( mLayerPolygonZ->getFeature( 1 ).geometry().asWkt(), wkt5 );
 
   mLayerTopo->startEditing();
-  const QString wkt6 = "Polygon ((0 0, 4 0, 4 4, 0 4))";
+  const QString wkt6 = "Polygon ((0 0, 4 0, 4 4, 0 4, 0 0))";
   QgsFeature f6;
   f6.setGeometry( QgsGeometry::fromWkt( wkt6 ) );
-  const QString wkt7 = "Polygon ((7 0, 8 0, 8 4, 7 4))";
+  const QString wkt7 = "Polygon ((7 0, 8 0, 8 4, 7 4, 7 0))";
   QgsFeature f7;
   f7.setGeometry( QgsGeometry::fromWkt( wkt7 ) );
   QgsFeatureList flistTopo;
@@ -162,10 +170,24 @@ void TestQgsMapToolReshape::initTestCase()
 
   mLayerTopo2->startEditing();
   QgsFeature f8;
-  f8.setGeometry( QgsGeometry::fromWkt( QStringLiteral( "Polygon ((0 5, 4 5, 4 7, 0 7))" ) ) );
+  f8.setGeometry( QgsGeometry::fromWkt( QStringLiteral( "Polygon ((0 5, 4 5, 4 7, 0 7, 0 5))" ) ) );
   mLayerTopo2->dataProvider()->addFeatures( QgsFeatureList() << f8 );
   QCOMPARE( mLayerTopo2->featureCount(), 1 );
-  QCOMPARE( mLayerTopo2->getFeature( 1 ).geometry().asWkt(), QStringLiteral( "Polygon ((0 5, 4 5, 4 7, 0 7))" ) );
+  QCOMPARE( mLayerTopo2->getFeature( 1 ).geometry().asWkt(), QStringLiteral( "Polygon ((0 5, 4 5, 4 7, 0 7, 0 5))" ) );
+
+  mLayerLine->startEditing();
+  const QString wkt9 = QStringLiteral( "LineString (0 0, 10 10)" );
+  const QString wkt10 = QStringLiteral( "LineString (2 8, 0 6)" );
+  const QString wkt11 = QStringLiteral( "LineString (13 0, 13 8, 19 11, 25 8, 25 0, 13 0)" ); // cw oriented linestring ring
+  QgsFeature f9, f10, f11;
+  f9.setGeometry( QgsGeometry::fromWkt( wkt9 ) );
+  f10.setGeometry( QgsGeometry::fromWkt( wkt10 ) );
+  f11.setGeometry( QgsGeometry::fromWkt( wkt11 ) );
+  mLayerLine->dataProvider()->addFeatures( QgsFeatureList() << f9 << f10 << f11 );
+  QCOMPARE( mLayerLine->featureCount(), ( long ) 3 );
+  QCOMPARE( mLayerLine->getFeature( 1 ).geometry().asWkt(), wkt9 );
+  QCOMPARE( mLayerLine->getFeature( 2 ).geometry().asWkt(), wkt10 );
+  QCOMPARE( mLayerLine->getFeature( 3 ).geometry().asWkt(), wkt11 );
 
   QgsSnappingConfig cfg = mCanvas->snappingUtils()->config();
   cfg.setMode( Qgis::SnappingMode::AllLayers );
@@ -198,6 +220,25 @@ void TestQgsMapToolReshape::cleanupTestCase()
   QgsApplication::exitQgis();
 }
 
+void TestQgsMapToolReshape::testReshapeNoChange()
+{
+  TestQgsMapToolAdvancedDigitizingUtils utils( mCaptureTool );
+  // no snapping for this test
+  QgsSnappingConfig cfg = mCanvas->snappingUtils()->config();
+  cfg.setEnabled( false );
+  mCanvas->snappingUtils()->setConfig( cfg );
+
+  utils.mouseClick( 2, 2, Qt::LeftButton );
+  utils.mouseClick( 3, 2, Qt::LeftButton );
+  utils.mouseClick( 3, 2, Qt::RightButton );
+
+  // activate back snapping
+  cfg.setEnabled( true );
+  mCanvas->snappingUtils()->setConfig( cfg );
+
+  QCOMPARE( mLayerLineZ->undoStack()->index(), 0 );
+}
+
 void TestQgsMapToolReshape::testReshapeZ()
 {
   TestQgsMapToolAdvancedDigitizingUtils utils( mCaptureTool );
@@ -247,25 +288,61 @@ void TestQgsMapToolReshape::testTopologicalEditing()
   QgsSettingsRegistryCore::settingsDigitizingDefaultZValue->setValue( 333 );
 
   utils.mouseClick( 4, 4, Qt::LeftButton, Qt::KeyboardModifiers(), true );
-  utils.mouseClick( 7, 2, Qt::LeftButton, Qt::KeyboardModifiers(), true );
+  utils.mouseClick( 8, 2, Qt::LeftButton, Qt::KeyboardModifiers(), true );
   utils.mouseClick( 4, 0, Qt::LeftButton, Qt::KeyboardModifiers(), true );
   utils.mouseClick( 4, 0, Qt::RightButton );
 
   const QString wkt = "Polygon ((4 0, 8 2, 4 4, 0 4, 0 0, 4 0))";
-  const QString wkt2 = "Polygon ((7 0, 8 0, 8 2, 8 4, 7 4))";
+  const QString wkt2 = "Polygon ((7 0, 8 0, 8 2, 8 4, 7 4, 7 0))";
 
   QCOMPARE( mLayerTopo->getFeature( 1 ).geometry().asWkt(), wkt );
   QCOMPARE( mLayerTopo->getFeature( 2 ).geometry().asWkt(), wkt2 );
 
   mLayerTopo->undoStack()->undo();
 
-  QCOMPARE( mLayerTopo2->getFeature( 1 ).geometry().asWkt(), QStringLiteral( "Polygon ((0 5, 4 5, 4 7, 0 7))" ) );
-  QCOMPARE( mLayerTopo->getFeature( 1 ).geometry().asWkt(), QStringLiteral( "Polygon ((0 0, 4 0, 4 4, 0 4))" ) );
-  QCOMPARE( mLayerTopo->getFeature( 2 ).geometry().asWkt(), QStringLiteral( "Polygon ((7 0, 8 0, 8 4, 7 4))" ) );
+  QCOMPARE( mLayerTopo2->getFeature( 1 ).geometry().asWkt(), QStringLiteral( "Polygon ((0 5, 4 5, 4 7, 0 7, 0 5))" ) );
+  QCOMPARE( mLayerTopo->getFeature( 1 ).geometry().asWkt(), QStringLiteral( "Polygon ((0 0, 4 0, 4 4, 0 4, 0 0))" ) );
+  QCOMPARE( mLayerTopo->getFeature( 2 ).geometry().asWkt(), QStringLiteral( "Polygon ((7 0, 8 0, 8 4, 7 4, 7 0))" ) );
 
   QgsProject::instance()->setTopologicalEditing( topologicalEditing );
 }
 
+void TestQgsMapToolReshape::testTopologicalEditingNoSnap()
+{
+  QgsProject::instance()->addMapLayers( QList<QgsMapLayer *>() << mLayerTopo );
+  mCanvas->setLayers( QList<QgsMapLayer *>() << mLayerTopo );
+
+  QgsSnappingConfig cfg = mCanvas->snappingUtils()->config();
+  cfg.setEnabled( false );
+  mCanvas->snappingUtils()->setConfig( cfg );
+
+  const bool topologicalEditing = QgsProject::instance()->topologicalEditing();
+  QgsProject::instance()->setTopologicalEditing( true );
+  mCanvas->setCurrentLayer( mLayerTopo );
+  TestQgsMapToolAdvancedDigitizingUtils utils( mCaptureTool );
+
+  utils.mouseClick( 4, 4, Qt::LeftButton, Qt::KeyboardModifiers(), true );
+  utils.mouseClick( 8, 2, Qt::LeftButton, Qt::KeyboardModifiers(), true );
+  utils.mouseClick( 4, 0, Qt::LeftButton, Qt::KeyboardModifiers(), true );
+  utils.mouseClick( 4, 0, Qt::RightButton );
+
+  const QString wkt = "Polygon ((4 0, 8 2, 4 4, 0 4, 0 0, 4 0))";
+  const QString wkt2 = "Polygon ((7 0, 8 0, 8 2, 8 4, 7 4, 7 0))";
+
+  QCOMPARE( mLayerTopo->getFeature( 1 ).geometry().asWkt(), wkt );
+  QCOMPARE( mLayerTopo->getFeature( 2 ).geometry().asWkt(), wkt2 );
+
+  mLayerTopo->undoStack()->undo();
+
+  QCOMPARE( mLayerTopo2->getFeature( 1 ).geometry().asWkt(), QStringLiteral( "Polygon ((0 5, 4 5, 4 7, 0 7, 0 5))" ) );
+  QCOMPARE( mLayerTopo->getFeature( 1 ).geometry().asWkt(), QStringLiteral( "Polygon ((0 0, 4 0, 4 4, 0 4, 0 0))" ) );
+  QCOMPARE( mLayerTopo->getFeature( 2 ).geometry().asWkt(), QStringLiteral( "Polygon ((7 0, 8 0, 8 4, 7 4, 7 0))" ) );
+
+  QgsProject::instance()->setTopologicalEditing( topologicalEditing );
+  cfg.setEnabled( true );
+  mCanvas->snappingUtils()->setConfig( cfg );
+}
+
 void TestQgsMapToolReshape::testAvoidIntersectionAndTopoEdit()
 {
   QList<QgsMapLayer *> layers = { mLayerTopo, mLayerTopo2 };
@@ -292,15 +369,15 @@ void TestQgsMapToolReshape::testAvoidIntersectionAndTopoEdit()
   utils.mouseClick( 4, 5, Qt::RightButton );
 
   QCOMPARE( mLayerTopo2->getFeature( 1 ).geometry().asWkt(), QStringLiteral( "Polygon ((0 5, 0 7, 4 7, 4 5, 3 4, 1 4, 0 5))" ) );
-  QCOMPARE( mLayerTopo->getFeature( 1 ).geometry().asWkt(), QStringLiteral( "Polygon ((0 0, 4 0, 4 4, 3 4, 1 4, 0 4))" ) );
-  QCOMPARE( mLayerTopo->getFeature( 2 ).geometry().asWkt(), QStringLiteral( "Polygon ((7 0, 8 0, 8 4, 7 4))" ) );
+  QCOMPARE( mLayerTopo->getFeature( 1 ).geometry().asWkt(), QStringLiteral( "Polygon ((0 0, 4 0, 4 4, 3 4, 1 4, 0 4, 0 0))" ) );
+  QCOMPARE( mLayerTopo->getFeature( 2 ).geometry().asWkt(), QStringLiteral( "Polygon ((7 0, 8 0, 8 4, 7 4, 7 0))" ) );
 
   mLayerTopo2->undoStack()->undo();
   mLayerTopo->undoStack()->undo();
 
-  QCOMPARE( mLayerTopo2->getFeature( 1 ).geometry().asWkt(), QStringLiteral( "Polygon ((0 5, 4 5, 4 7, 0 7))" ) );
-  QCOMPARE( mLayerTopo->getFeature( 1 ).geometry().asWkt(), QStringLiteral( "Polygon ((0 0, 4 0, 4 4, 0 4))" ) );
-  QCOMPARE( mLayerTopo->getFeature( 2 ).geometry().asWkt(), QStringLiteral( "Polygon ((7 0, 8 0, 8 4, 7 4))" ) );
+  QCOMPARE( mLayerTopo2->getFeature( 1 ).geometry().asWkt(), QStringLiteral( "Polygon ((0 5, 4 5, 4 7, 0 7, 0 5))" ) );
+  QCOMPARE( mLayerTopo->getFeature( 1 ).geometry().asWkt(), QStringLiteral( "Polygon ((0 0, 4 0, 4 4, 0 4, 0 0))" ) );
+  QCOMPARE( mLayerTopo->getFeature( 2 ).geometry().asWkt(), QStringLiteral( "Polygon ((7 0, 8 0, 8 4, 7 4, 7 0))" ) );
 
   QgsProject::instance()->setTopologicalEditing( topologicalEditing );
   QgsProject::instance()->setAvoidIntersectionsMode( mode );
@@ -381,7 +458,7 @@ void TestQgsMapToolReshape::testWithTracing()
   QgsProject::instance()->addMapLayers( QList<QgsMapLayer *>() << mLayerTopo );
   mCanvas->setLayers( QList<QgsMapLayer *>() << mLayerTopo );
 
-  std::unique_ptr<QgsMapCanvasTracer> tracer( new QgsMapCanvasTracer( mCanvas, nullptr ) );
+  auto tracer = std::make_unique<QgsMapCanvasTracer>( mCanvas, nullptr );
 
   const bool topologicalEditing = QgsProject::instance()->topologicalEditing();
   QgsProject::instance()->setTopologicalEditing( true );
@@ -397,17 +474,68 @@ void TestQgsMapToolReshape::testWithTracing()
   utils.mouseClick( 7, 4, Qt::LeftButton, Qt::KeyboardModifiers(), true );
   utils.mouseClick( 8, 5, Qt::RightButton );
 
-  QCOMPARE( mLayerTopo->getFeature( 1 ).geometry().asWkt(), "Polygon ((0 0, 4 0, 4 4, 0 4))" );
+  QCOMPARE( mLayerTopo->getFeature( 1 ).geometry().asWkt(), "Polygon ((0 0, 4 0, 4 4, 0 4, 0 0))" );
   QCOMPARE( mLayerTopo->getFeature( 2 ).geometry().asWkt(), "Polygon ((7 0, 8 0, 8 4, 7 4, 4 4, 4 0, 7 0))" );
 
   mLayerTopo->undoStack()->undo();
 
-  QCOMPARE( mLayerTopo->getFeature( 1 ).geometry().asWkt(), QStringLiteral( "Polygon ((0 0, 4 0, 4 4, 0 4))" ) );
-  QCOMPARE( mLayerTopo->getFeature( 2 ).geometry().asWkt(), QStringLiteral( "Polygon ((7 0, 8 0, 8 4, 7 4))" ) );
+  QCOMPARE( mLayerTopo->getFeature( 1 ).geometry().asWkt(), QStringLiteral( "Polygon ((0 0, 4 0, 4 4, 0 4, 0 0))" ) );
+  QCOMPARE( mLayerTopo->getFeature( 2 ).geometry().asWkt(), QStringLiteral( "Polygon ((7 0, 8 0, 8 4, 7 4, 7 0))" ) );
 
   QgsProject::instance()->setTopologicalEditing( topologicalEditing );
 }
 
+void TestQgsMapToolReshape::testKeepDirection()
+{
+  TestQgsMapToolAdvancedDigitizingUtils utils( mCaptureTool );
+  mCanvas->setCurrentLayer( mLayerLine );
+
+  // no snapping for this test
+  QgsSnappingConfig cfg = mCanvas->snappingUtils()->config();
+  cfg.setEnabled( false );
+  mCanvas->snappingUtils()->setConfig( cfg );
+
+  // intersect linestring 4 times, and go backwards compared to its direction
+  utils.mouseClick( 9, 8, Qt::LeftButton );
+  utils.mouseClick( 9, 10, Qt::LeftButton );
+  utils.mouseClick( 8, 9, Qt::LeftButton );
+  utils.mouseClick( 8, 7, Qt::LeftButton );
+  utils.mouseClick( 7, 6, Qt::LeftButton );
+  utils.mouseClick( 7, 8, Qt::LeftButton );
+  utils.mouseClick( 6, 7, Qt::LeftButton );
+  utils.mouseClick( 6, 5, Qt::LeftButton );
+  utils.mouseClick( 6, 5, Qt::RightButton );
+
+  QString wkt1 = QStringLiteral( "LineString (0 0, 6 6, 6 7, 7 8, 7 7, 7 6, 8 7, 8 8, 8 9, 9 10, 9 9, 10 10)" );
+  QCOMPARE( mLayerLine->getFeature( 1 ).geometry().asWkt(), wkt1 );
+
+  // extend linestring from its start point
+  utils.mouseClick( 2, 8, Qt::LeftButton );
+  utils.mouseClick( 3, 9, Qt::LeftButton );
+  utils.mouseClick( 3, 9, Qt::RightButton );
+
+  QString wkt2 = QStringLiteral( "LineString (3 9, 2 8, 0 6)" );
+  QCOMPARE( mLayerLine->getFeature( 2 ).geometry().asWkt(), wkt2 );
+
+  // intersect linestring ring 4 times, and go backwards compared to its direction
+  utils.mouseClick( 14, 7, Qt::LeftButton );
+  utils.mouseClick( 12, 7, Qt::LeftButton );
+  utils.mouseClick( 12, 5, Qt::LeftButton );
+  utils.mouseClick( 14, 5, Qt::LeftButton );
+  utils.mouseClick( 14, 3, Qt::LeftButton );
+  utils.mouseClick( 12, 3, Qt::LeftButton );
+  utils.mouseClick( 12, 1, Qt::LeftButton );
+  utils.mouseClick( 14, 1, Qt::LeftButton );
+  utils.mouseClick( 14, 1, Qt::RightButton );
+
+  QString wkt3 = QStringLiteral( "LineString (13 1, 12 1, 12 3, 13 3, 14 3, 14 5, 13 5, 12 5, 12 7, 13 7, 13 8, 19 11, 25 8, 25 0, 13 0, 13 1)" );
+  QCOMPARE( mLayerLine->getFeature( 3 ).geometry().asWkt(), wkt3 );
+
+  // activate back snapping
+  cfg.setEnabled( true );
+  mCanvas->snappingUtils()->setConfig( cfg );
+}
+
 
 QGSTEST_MAIN( TestQgsMapToolReshape )
 #include "testqgsmaptoolreshape.moc"
