@@ -67,6 +67,7 @@ class TestQgsVectorTileLayer : public QgsTest
     void testMbtilesProviderMetadata();
     void test_relativePathsMbTiles();
     void test_absoluteRelativeUriMbTiles();
+    void test_mbtilesZoom16();
 
     void test_relativePathsXyz();
     void test_absoluteRelativeUriXyz();
@@ -109,13 +110,13 @@ void TestQgsVectorTileLayer::initTestCase()
   QColor polygonFillColor = Qt::blue;
   const QColor polygonStrokeColor = polygonFillColor;
   polygonFillColor.setAlpha( 100 );
-  const double polygonStrokeWidth = DEFAULT_LINE_WIDTH * 2;
+  const double polygonStrokeWidth = Qgis::DEFAULT_LINE_WIDTH * 2;
   const QColor lineStrokeColor = Qt::blue;
-  const double lineStrokeWidth = DEFAULT_LINE_WIDTH * 2;
+  const double lineStrokeWidth = Qgis::DEFAULT_LINE_WIDTH * 2;
   QColor pointFillColor = Qt::red;
   const QColor pointStrokeColor = pointFillColor;
   pointFillColor.setAlpha( 100 );
-  const double pointSize = DEFAULT_POINT_SIZE;
+  const double pointSize = Qgis::DEFAULT_POINT_SIZE;
 
   QgsVectorTileBasicRenderer *rend = new QgsVectorTileBasicRenderer;
   rend->setStyles( QgsVectorTileBasicRenderer::simpleStyle(
@@ -200,7 +201,7 @@ void TestQgsVectorTileLayer::test_labeling()
   // use a different renderer to make the labels stand out more
   QgsVectorTileBasicRenderer *rend = new QgsVectorTileBasicRenderer;
   rend->setStyles( QgsVectorTileBasicRenderer::simpleStyle(
-    Qt::transparent, Qt::white, DEFAULT_LINE_WIDTH * 2,
+    Qt::transparent, Qt::white, Qgis::DEFAULT_LINE_WIDTH * 2,
     Qt::transparent, 0,
     Qt::transparent, Qt::transparent, 0
   ) );
@@ -331,7 +332,7 @@ void TestQgsVectorTileLayer::test_relativePathsMbTiles()
 
   const QString srcMbtiles = QStringLiteral( "type=mbtiles&url=%1/vector_tile/mbtiles_vt.mbtiles" ).arg( TEST_DATA_DIR );
 
-  std::unique_ptr<QgsVectorTileLayer> layer = std::make_unique<QgsVectorTileLayer>( srcMbtiles );
+  auto layer = std::make_unique<QgsVectorTileLayer>( srcMbtiles );
   QVERIFY( layer->isValid() );
   QCOMPARE( layer->providerType(), QStringLiteral( "mbtilesvectortiles" ) );
 
@@ -371,6 +372,17 @@ void TestQgsVectorTileLayer::test_absoluteRelativeUriMbTiles()
   QCOMPARE( vectorTileMetadata->relativeToAbsoluteUri( relativeUri, context ), absoluteUri );
 }
 
+void TestQgsVectorTileLayer::test_mbtilesZoom16()
+{
+  const QString srcMbtiles = QStringLiteral( "type=mbtiles&url=%1/vector_tile/z16.mbtiles" ).arg( TEST_DATA_DIR );
+
+  auto layer = std::make_unique<QgsVectorTileLayer>( srcMbtiles );
+  QVERIFY( layer->isValid() );
+  QCOMPARE( layer->providerType(), QStringLiteral( "mbtilesvectortiles" ) );
+  QCOMPARE( layer->sourceMinZoom(), 16 );
+  QCOMPARE( layer->sourceMaxZoom(), 16 );
+}
+
 void TestQgsVectorTileLayer::test_relativePathsXyz()
 {
   QgsReadWriteContext contextRel;
@@ -380,7 +392,7 @@ void TestQgsVectorTileLayer::test_relativePathsXyz()
   const QString srcXyzLocal = "type=xyz&url=file:///home/qgis/%7Bz%7D/%7Bx%7D/%7By%7D.pbf";
   const QString srcXyzRemote = "type=xyz&url=http://www.example.com/%7Bz%7D/%7Bx%7D/%7By%7D.pbf";
 
-  std::unique_ptr<QgsVectorTileLayer> layer = std::make_unique<QgsVectorTileLayer>( srcXyzLocal );
+  auto layer = std::make_unique<QgsVectorTileLayer>( srcXyzLocal );
   QCOMPARE( layer->providerType(), QStringLiteral( "xyzvectortiles" ) );
 
   // encode source: converting absolute paths to relative
@@ -487,7 +499,7 @@ void TestQgsVectorTileLayer::test_relativePathsVtpk()
 
   const QString srcVtpk = QStringLiteral( "type=vtpk&url=%1/testvtpk.vtpk" ).arg( TEST_DATA_DIR );
 
-  std::unique_ptr<QgsVectorTileLayer> layer = std::make_unique<QgsVectorTileLayer>( srcVtpk );
+  auto layer = std::make_unique<QgsVectorTileLayer>( srcVtpk );
   QVERIFY( layer->isValid() );
   QCOMPARE( layer->providerType(), QStringLiteral( "vtpkvectortiles" ) );
 
@@ -534,13 +546,13 @@ void TestQgsVectorTileLayer::test_polygonWithLineStyle()
   ds.setParam( "type", "xyz" );
   ds.setParam( "url", QString( "file://%1/{z}-{x}-{y}.pbf" ).arg( mDataDir ) );
   ds.setParam( "zmax", "1" );
-  std::unique_ptr<QgsVectorTileLayer> layer = std::make_unique<QgsVectorTileLayer>( ds.encodedUri(), "Vector Tiles Test" );
+  auto layer = std::make_unique<QgsVectorTileLayer>( ds.encodedUri(), "Vector Tiles Test" );
   QVERIFY( layer->isValid() );
 
   mMapSettings->setLayers( QList<QgsMapLayer *>() << layer.get() );
 
   const QColor lineStrokeColor = Qt::blue;
-  const double lineStrokeWidth = DEFAULT_LINE_WIDTH * 2;
+  const double lineStrokeWidth = Qgis::DEFAULT_LINE_WIDTH * 2;
 
   QgsSimpleLineSymbolLayer *lineSymbolLayer = new QgsSimpleLineSymbolLayer;
   lineSymbolLayer->setColor( lineStrokeColor );
@@ -574,7 +586,7 @@ void TestQgsVectorTileLayer::test_polygonWithMarker()
   ds.setParam( "type", "xyz" );
   ds.setParam( "url", QString( "file://%1/{z}-{x}-{y}.pbf" ).arg( mDataDir ) );
   ds.setParam( "zmax", "1" );
-  std::unique_ptr<QgsVectorTileLayer> layer = std::make_unique<QgsVectorTileLayer>( ds.encodedUri(), "Vector Tiles Test" );
+  auto layer = std::make_unique<QgsVectorTileLayer>( ds.encodedUri(), "Vector Tiles Test" );
   QVERIFY( layer->isValid() );
 
   mMapSettings->setLayers( QList<QgsMapLayer *>() << layer.get() );
@@ -612,13 +624,13 @@ void TestQgsVectorTileLayer::test_styleMinZoomBeyondTileMaxZoom()
   ds.setParam( "type", "xyz" );
   ds.setParam( "url", QString( "file://%1/{z}-{x}-{y}.pbf" ).arg( mDataDir ) );
   ds.setParam( "zmax", "1" );
-  std::unique_ptr<QgsVectorTileLayer> layer = std::make_unique<QgsVectorTileLayer>( ds.encodedUri(), "Vector Tiles Test" );
+  auto layer = std::make_unique<QgsVectorTileLayer>( ds.encodedUri(), "Vector Tiles Test" );
   QVERIFY( layer->isValid() );
 
   mMapSettings->setLayers( QList<QgsMapLayer *>() << layer.get() );
 
   const QColor lineStrokeColor = Qt::blue;
-  const double lineStrokeWidth = DEFAULT_LINE_WIDTH * 2;
+  const double lineStrokeWidth = Qgis::DEFAULT_LINE_WIDTH * 2;
 
   QgsSimpleLineSymbolLayer *lineSymbolLayer = new QgsSimpleLineSymbolLayer;
   lineSymbolLayer->setColor( lineStrokeColor );
@@ -652,13 +664,13 @@ void TestQgsVectorTileLayer::test_filterRuleAllLayers()
   QgsDataSourceUri ds;
   ds.setParam( "type", "mbtiles" );
   ds.setParam( "url", QString( "/%1/mbtiles_vt.mbtiles" ).arg( mDataDir ) );
-  std::unique_ptr<QgsVectorTileLayer> layer = std::make_unique<QgsVectorTileLayer>( ds.encodedUri(), "Vector Tiles Test" );
+  auto layer = std::make_unique<QgsVectorTileLayer>( ds.encodedUri(), "Vector Tiles Test" );
   QVERIFY( layer->isValid() );
 
   mMapSettings->setLayers( QList<QgsMapLayer *>() << layer.get() );
 
   const QColor lineStrokeColor = Qt::blue;
-  const double lineStrokeWidth = DEFAULT_LINE_WIDTH * 4;
+  const double lineStrokeWidth = Qgis::DEFAULT_LINE_WIDTH * 4;
 
   QgsSimpleLineSymbolLayer *lineSymbolLayer = new QgsSimpleLineSymbolLayer;
   lineSymbolLayer->setColor( lineStrokeColor );
